@@ -32,7 +32,7 @@
 #include <folly/FileUtil.h>
 #include <folly/MapUtil.h>
 #include <folly/String.h>
-#include <folly/container/Foreach.h>
+#include <folly/detail/PerfScoped.h>
 #include <folly/json.h>
 
 // This needs to be at the end because some versions end up including
@@ -50,6 +50,17 @@ FOLLY_GFLAGS_DEFINE_bool(json, false, "Output in JSON format.");
 
 FOLLY_GFLAGS_DEFINE_bool(bm_estimate_time, false, "Estimate running time");
 
+#if FOLLY_PERF_IS_SUPPORTED
+FOLLY_GFLAGS_DEFINE_string(
+    bm_perf_args,
+    "",
+    "Run selected benchmarks while attaching `perf` profiling tool."
+    "Advantage over attaching perf externally is that this skips "
+    "initialization. The first iteration of the benchmark is also "
+    "skipped to allow for all statics to be set up. This requires perf "
+    " to be available on the system. Example: --bm_perf_args=\"record -g\"");
+#endif
+
 FOLLY_GFLAGS_DEFINE_bool(
     bm_profile, false, "Run benchmarks with constant number of iterations");
 
@@ -61,6 +72,11 @@ FOLLY_GFLAGS_DEFINE_string(
     "",
     "Print benchmark results relative to an earlier dump (via --bm_json_verbose)");
 
+FOLLY_GFLAGS_DEFINE_bool(
+    bm_warm_up_iteration,
+    false,
+    "Run one iteration of the benchmarks before measuring. Always true if `bm_perf_args` is passed");
+
 FOLLY_GFLAGS_DEFINE_string(
     bm_json_verbose,
     "",
@@ -95,19 +111,23 @@ FOLLY_GFLAGS_DEFINE_uint32(
     "Maximum number of trials (iterations) executed for each benchmark.");
 
 namespace folly {
+namespace detail {
 
-std::chrono::high_resolution_clock::duration BenchmarkSuspender::timeSpent;
-
-typedef function<detail::TimeIterData(unsigned int)> BenchmarkFun;
-
-vector<detail::BenchmarkRegistration>& benchmarks() {
-  static vector<detail::BenchmarkRegistration> _benchmarks;
-  return _benchmarks;
+BenchmarkingState<std::chrono::high_resolution_clock>& globalBenchmarkState() {
+  static detail::BenchmarkingState<std::chrono::high_resolution_clock> state;
+  return state;
 }
 
+} // namespace detail
+
+using BenchmarkFun = std::function<detail::TimeIterData(unsigned int)>;
+
 #define FB_FOLLY_GLOBAL_BENCHMARK_BASELINE fbFollyGlobalBenchmarkBaseline
 #define FB_STRINGIZE_X2(x) FOLLY_PP_STRINGIZE(x)
 
+constexpr const char kGlobalBenchmarkBaseline[] =
+    FB_STRINGIZE_X2(FB_FOLLY_GLOBAL_BENCHMARK_BASELINE);
+
 // Add the global baseline
 BENCHMARK(FB_FOLLY_GLOBAL_BENCHMARK_BASELINE) {
 #ifdef _MSC_VER
@@ -117,26 +137,9 @@ BENCHMARK(FB_FOLLY_GLOBAL_BENCHMARK_BASELINE) {
 #endif
 }
 
-size_t getGlobalBenchmarkBaselineIndex() {
-  const char* global = FB_STRINGIZE_X2(FB_FOLLY_GLOBAL_BENCHMARK_BASELINE);
-  auto it = std::find_if(
-      benchmarks().begin(),
-      benchmarks().end(),
-      [global](const detail::BenchmarkRegistration& v) {
-        return v.name == global;
-      });
-  CHECK(it != benchmarks().end());
-  return size_t(std::distance(benchmarks().begin(), it));
-}
-
 #undef FB_STRINGIZE_X2
 #undef FB_FOLLY_GLOBAL_BENCHMARK_BASELINE
 
-void detail::addBenchmarkImpl(
-    const char* file, StringPiece name, BenchmarkFun fun, bool useCounter) {
-  benchmarks().push_back({file, name.str(), std::move(fun), useCounter});
-}
-
 static std::pair<double, UserCounters> runBenchmarkGetNSPerIteration(
     const BenchmarkFun& fun, const double globalBaseline) {
   using std::chrono::duration_cast;
@@ -652,42 +655,82 @@ void checkRunMode() {
 
 namespace {
 
-std::pair<std::set<std::string>, std::vector<detail::BenchmarkResult>>
-runBenchmarksWithPrinter(BenchmarkResultsPrinter* FOLLY_NULLABLE printer) {
-  vector<detail::BenchmarkResult> results;
-  results.reserve(benchmarks().size() - 1);
+struct BenchmarksToRun {
+  const detail::BenchmarkRegistration* baseline = nullptr;
+  std::vector<const detail::BenchmarkRegistration*> benchmarks;
+};
+
+BenchmarksToRun selectBenchmarksToRun(
+    const std::vector<detail::BenchmarkRegistration>& benchmarks) {
+  BenchmarksToRun res;
+
+  folly::Optional<boost::regex> bmRegex;
+
+  res.benchmarks.reserve(benchmarks.size());
 
-  std::unique_ptr<boost::regex> bmRegex;
   if (!FLAGS_bm_regex.empty()) {
-    bmRegex = std::make_unique<boost::regex>(FLAGS_bm_regex);
+    bmRegex.emplace(FLAGS_bm_regex);
   }
 
+  for (auto& bm : benchmarks) {
+    if (bm.name == "-") { // skip separators
+      continue;
+    }
+
+    if (bm.name == kGlobalBenchmarkBaseline) {
+      res.baseline = &bm;
+      continue;
+    }
+
+    if (!bmRegex || boost::regex_search(bm.name, *bmRegex)) {
+      res.benchmarks.push_back(&bm);
+    }
+  }
+
+  CHECK(res.baseline);
+
+  return res;
+}
+
+void maybeRunWarmUpIteration(const BenchmarksToRun& toRun) {
+  bool shouldRun = FLAGS_bm_warm_up_iteration;
+
+#if FOLLY_PERF_IS_SUPPORTED
+  shouldRun = shouldRun || !FLAGS_bm_perf_args.empty();
+#endif
+
+  if (!shouldRun) {
+    return;
+  }
+
+  for (const auto* bm : toRun.benchmarks) {
+    bm->func(1);
+  }
+}
+
+std::pair<std::set<std::string>, std::vector<detail::BenchmarkResult>>
+runBenchmarksWithPrinterImpl(
+    BenchmarkResultsPrinter* FOLLY_NULLABLE printer,
+    const BenchmarksToRun& toRun) {
+  vector<detail::BenchmarkResult> results;
+  results.reserve(toRun.benchmarks.size());
+
   // PLEASE KEEP QUIET. MEASUREMENTS IN PROGRESS.
 
-  size_t baselineIndex = getGlobalBenchmarkBaselineIndex();
-
   auto const globalBaseline =
-      runBenchmarkGetNSPerIteration(benchmarks()[baselineIndex].func, 0);
+      runBenchmarkGetNSPerIteration(toRun.baseline->func, 0);
 
   std::set<std::string> counterNames;
-  FOR_EACH_RANGE (i, 0, benchmarks().size()) {
-    if (i == baselineIndex) {
-      continue;
-    }
+  for (const auto bmPtr : toRun.benchmarks) {
     std::pair<double, UserCounters> elapsed;
-    auto& bm = benchmarks()[i];
-    if (bm.name != "-") { // skip separators
-      if (bmRegex && !boost::regex_search(bm.name, *bmRegex)) {
-        continue;
-      }
-      if (FLAGS_bm_profile) {
-        elapsed = runProfilingGetNSPerIteration(bm.func, globalBaseline.first);
-      } else {
-        elapsed = FLAGS_bm_estimate_time
-            ? runBenchmarkGetNSPerIterationEstimate(
-                  bm.func, globalBaseline.first)
-            : runBenchmarkGetNSPerIteration(bm.func, globalBaseline.first);
-      }
+    const detail::BenchmarkRegistration& bm = *bmPtr;
+
+    if (FLAGS_bm_profile) {
+      elapsed = runProfilingGetNSPerIteration(bm.func, globalBaseline.first);
+    } else {
+      elapsed = FLAGS_bm_estimate_time
+          ? runBenchmarkGetNSPerIterationEstimate(bm.func, globalBaseline.first)
+          : runBenchmarkGetNSPerIteration(bm.func, globalBaseline.first);
     }
 
     // if customized user counters is used, it cannot print the result in real
@@ -732,15 +775,81 @@ bool writeResultsToFile(
 
 namespace detail {
 
+std::ostream& operator<<(std::ostream& os, const BenchmarkResult& x) {
+  folly::dynamic r;
+  benchmarkResultsToDynamic({x}, r);
+  return os << r[0];
+}
+
+bool operator==(const BenchmarkResult& x, const BenchmarkResult& y) {
+  auto xtime = static_cast<std::uint64_t>(x.timeInNs * 1000);
+  auto ytime = static_cast<std::uint64_t>(y.timeInNs * 1000);
+  return x.name == y.name && x.file == y.file && xtime == ytime &&
+      x.counters == y.counters;
+}
+
+std::chrono::high_resolution_clock::duration BenchmarkSuspenderBase::timeSpent;
+
+void BenchmarkingStateBase::addBenchmarkImpl(
+    const char* file, StringPiece name, BenchmarkFun fun, bool useCounter) {
+  std::lock_guard<std::mutex> guard(mutex_);
+  benchmarks_.push_back({file, name.str(), std::move(fun), useCounter});
+}
+
+bool BenchmarkingStateBase::useCounters() const {
+  std::lock_guard<std::mutex> guard(mutex_);
+  return std::any_of(
+      benchmarks_.begin(), benchmarks_.end(), [](const auto& bm) {
+        return bm.useCounter;
+      });
+}
+
+// static
+folly::StringPiece BenchmarkingStateBase::getGlobalBaselineNameForTests() {
+  return kGlobalBenchmarkBaseline;
+}
+
+PerfScoped BenchmarkingStateBase::doSetUpPerfScoped(
+    const std::vector<std::string>& args) const {
+  return PerfScoped{args};
+}
+
+PerfScoped BenchmarkingStateBase::setUpPerfScoped() const {
+  std::vector<std::string> perfArgs;
+#if FOLLY_PERF_IS_SUPPORTED
+  folly::split(' ', FLAGS_bm_perf_args, perfArgs, true);
+#endif
+  if (perfArgs.empty()) {
+    return PerfScoped{};
+  }
+  return doSetUpPerfScoped(perfArgs);
+}
+
+template <typename Printer>
+std::pair<std::set<std::string>, std::vector<BenchmarkResult>>
+BenchmarkingStateBase::runBenchmarksWithPrinter(Printer* printer) const {
+  std::lock_guard<std::mutex> guard(mutex_);
+  BenchmarksToRun toRun = selectBenchmarksToRun(benchmarks_);
+  maybeRunWarmUpIteration(toRun);
+
+  detail::PerfScoped perf = setUpPerfScoped();
+  return runBenchmarksWithPrinterImpl(printer, toRun);
+}
+
+std::vector<BenchmarkResult> BenchmarkingStateBase::runBenchmarksWithResults()
+    const {
+  return runBenchmarksWithPrinter(
+             static_cast<BenchmarkResultsPrinter*>(nullptr))
+      .second;
+}
+
 std::vector<BenchmarkResult> runBenchmarksWithResults() {
-  return runBenchmarksWithPrinter(nullptr).second;
+  return globalBenchmarkState().runBenchmarksWithResults();
 }
 
 } // namespace detail
 
 void runBenchmarks() {
-  CHECK(!benchmarks().empty());
-
   if (FLAGS_bm_profile) {
     printf(
         "WARNING: Running with constant number of iterations. Results might be jittery.\n");
@@ -748,17 +857,17 @@ void runBenchmarks() {
 
   checkRunMode();
 
+  auto& state = detail::globalBenchmarkState();
+
   BenchmarkResultsPrinter printer;
-  bool useCounter =
-      std::any_of(benchmarks().begin(), benchmarks().end(), [](const auto& bm) {
-        return bm.useCounter;
-      });
+  bool useCounter = state.useCounters();
+
   // PLEASE KEEP QUIET. MEASUREMENTS IN PROGRESS.
 
   const bool shouldPrintInline =
       FLAGS_bm_relative_to.empty() && !FLAGS_json && !useCounter;
   auto benchmarkResults =
-      runBenchmarksWithPrinter(shouldPrintInline ? &printer : nullptr);
+      state.runBenchmarksWithPrinter(shouldPrintInline ? &printer : nullptr);
 
   // PLEASE MAKE NOISE. MEASUREMENTS DONE.
 
