@@ -109,21 +109,6 @@ inline exception_wrapper::exception_wrapper(
     PrivateCtor, in_place_type_t<Ex>, As&&... as)
     : ptr_{std::make_exception_ptr(Ex(std::forward<As>(as)...))} {}
 
-template <class Ex>
-inline exception_wrapper::exception_wrapper(
-    from_catch_ref_t, std::exception_ptr const& ptr, Ex& ex) noexcept
-    : exception_wrapper{from_catch_ref_t{}, folly::copy(ptr), ex} {}
-
-template <class Ex>
-inline exception_wrapper::exception_wrapper(
-    from_catch_ref_t, std::exception_ptr&& ptr, Ex& ex) noexcept
-    : ptr_{std::move(ptr)} {
-  assert(ptr_);
-  (void)ex;
-  assert(exception_ptr_get_object<Ex>(ptr_));
-  assert(exception_ptr_get_object<Ex>(ptr_) == &ex || kIsWindows);
-}
-
 namespace exception_wrapper_detail {
 template <class Ex>
 Ex&& dont_slice(Ex&& ex) {
