@@ -27,12 +27,21 @@
 #define FOLLY_CPLUSPLUS __cplusplus
 #endif
 
+// On MSVC an incorrect <version> header get's picked up
+#if !defined(_MSC_VER) && __has_include(<version>)
+#include <version>
+#endif
+
 static_assert(FOLLY_CPLUSPLUS >= 201402L, "__cplusplus >= 201402L");
 
 #if defined(__GNUC__) && !defined(__clang__)
 static_assert(__GNUC__ >= 7, "__GNUC__ >= 7");
 #endif
 
+#if defined(_MSC_VER) || defined(_CPPLIB_VER)
+static_assert(FOLLY_CPLUSPLUS >= 201703L, "__cplusplus >= 201703L");
+#endif
+
 // Unaligned loads and stores
 namespace folly {
 #if defined(FOLLY_HAVE_UNALIGNED_ACCESS) && FOLLY_HAVE_UNALIGNED_ACCESS
@@ -64,18 +73,18 @@ constexpr bool kHasUnalignedAccess = false;
 // warn unused result
 #if defined(__has_cpp_attribute)
 #if __has_cpp_attribute(nodiscard)
+#if defined(__clang__) || defined(__GNUC__)
+#if __clang_major__ >= 10 || __GNUC__ >= 10
+// early clang and gcc both warn on [[nodiscard]] when applied to class ctors
+// easiest option is just to avoid emitting [[nodiscard]] under early clang/gcc
 #define FOLLY_NODISCARD [[nodiscard]]
 #endif
 #endif
-#if !defined FOLLY_NODISCARD
-#if defined(_MSC_VER) && (_MSC_VER >= 1700)
-#define FOLLY_NODISCARD _Check_return_
-#elif defined(__GNUC__)
-#define FOLLY_NODISCARD __attribute__((__warn_unused_result__))
-#else
-#define FOLLY_NODISCARD
 #endif
 #endif
+#ifndef FOLLY_NODISCARD
+#define FOLLY_NODISCARD
+#endif
 
 // target
 #ifdef _MSC_VER
@@ -115,12 +124,19 @@ constexpr bool kHasUnalignedAccess = false;
 #define FOLLY_S390X 0
 #endif
 
+#if defined(__riscv)
+#define FOLLY_RISCV64 1
+#else
+#define FOLLY_RISCV64 0
+#endif
+
 namespace folly {
 constexpr bool kIsArchArm = FOLLY_ARM == 1;
 constexpr bool kIsArchAmd64 = FOLLY_X64 == 1;
 constexpr bool kIsArchAArch64 = FOLLY_AARCH64 == 1;
 constexpr bool kIsArchPPC64 = FOLLY_PPC64 == 1;
 constexpr bool kIsArchS390X = FOLLY_S390X == 1;
+constexpr bool kIsArchRISCV64 = FOLLY_RISCV64 == 1;
 } // namespace folly
 
 namespace folly {
@@ -202,7 +218,6 @@ constexpr bool kIsSanitize = false;
 // MSVC specific defines
 // mainly for posix compat
 #ifdef _MSC_VER
-#include <folly/portability/SysTypes.h>
 
 // We have compiler support for the newest of the new, but
 // MSVC doesn't tell us that.
@@ -597,3 +612,16 @@ constexpr auto kCpplibVer = 0;
 #else
 #define FOLLY_CONSTEVAL constexpr
 #endif
+
+// C++17 deduction guides
+#if defined(__cpp_deduction_guides) && __cpp_deduction_guides >= 201703L
+#define FOLLY_HAS_DEDUCTION_GUIDES 1
+#else
+#define FOLLY_HAS_DEDUCTION_GUIDES 0
+#endif
+// C++20 ranges
+#if defined(__cpp_lib_ranges)
+#define FOLLY_HAS_RANGES 1
+#else
+#define FOLLY_HAS_RANGES 0
+#endif
