@@ -213,7 +213,7 @@ TEST(UnboundedQueue, peek) {
   peek_test<UMPSC, true>();
 }
 
-TEST(UnboundedQueue, cleanup_on_destruction) {
+TEST(UnboundedQueue, cleanupOnDestruction) {
   struct Foo {
     int* p_{nullptr};
     explicit Foo(int* p) : p_(p) {}
@@ -360,7 +360,7 @@ void enq_deq_test(const int nprod, const int ncons) {
   run_once(nprod, ncons, prod, cons, endfn);
 }
 
-TEST(UnboundedQueue, enq_deq) {
+TEST(UnboundedQueue, enqDeq) {
   /* SPSC */
   enq_deq_test<true, true, false>(1, 1);
   enq_deq_test<true, true, true>(1, 1);
@@ -453,12 +453,12 @@ uint64_t bench(const int nprod, const int ncons, const std::string& name) {
       for (uint64_t i = tid; i < ops; i += ncons) {
         T v;
         if (Op == 0 || Op == 3) {
-          while (UNLIKELY(!q.try_dequeue(v))) {
+          while (FOLLY_UNLIKELY(!q.try_dequeue(v))) {
             /* keep trying */;
           }
         } else if (Op == 1 || Op == 4) {
           auto duration = std::chrono::microseconds(1000);
-          while (UNLIKELY(!q.try_dequeue_for(v, duration))) {
+          while (FOLLY_UNLIKELY(!q.try_dequeue_for(v, duration))) {
             /* keep trying */;
           }
         } else {
