@@ -157,7 +157,7 @@ struct FOLLY_MSVC_DECLSPEC(empty_bases) BasePolicy
   // Detection for folly_assume_32bit_hash
 
   template <typename Hasher, typename Void = void>
-  struct ShouldAssume32BitHash : std::false_type {};
+  struct ShouldAssume32BitHash : bool_constant<!sizeof(Hasher)> {};
 
   template <typename Hasher>
   struct ShouldAssume32BitHash<
@@ -952,7 +952,7 @@ class VectorContainerIterator : public BaseIter<ValuePtr, uint32_t> {
   pointer operator->() const { return current_; }
 
   VectorContainerIterator& operator++() {
-    if (UNLIKELY(current_ == lowest_)) {
+    if (FOLLY_UNLIKELY(current_ == lowest_)) {
       current_ = nullptr;
     } else {
       --current_;
