@@ -29,10 +29,6 @@
 #include <utility>
 #include <vector>
 
-#if FOLLY_HAS_STRING_VIEW
-#include <string_view> // @manual
-#endif
-
 #include <folly/Bits.h>
 #include <folly/ConstexprMath.h>
 #include <folly/Likely.h>
@@ -53,6 +49,10 @@
 #include <folly/container/detail/F14IntrinsicsAvailability.h>
 #include <folly/container/detail/F14Mask.h>
 
+#if FOLLY_HAS_STRING_VIEW
+#include <string_view> // @manual
+#endif
+
 #if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 
 #if FOLLY_F14_CRC_INTRINSIC_AVAILABLE
@@ -69,7 +69,7 @@
 
 #if FOLLY_NEON
 #include <arm_neon.h> // uint8x16t intrinsics
-#else // SSE2
+#elif FOLLY_SSE >= 2 // SSE2
 #include <emmintrin.h> // _mm_set1_epi8
 #include <immintrin.h> // __m128i intrinsics
 #include <xmmintrin.h> // _mm_prefetch
@@ -154,7 +154,7 @@ struct StdNodeReplica {
 #else
 
 template <typename H>
-struct StdIsFastHash : std::true_type {};
+struct StdIsFastHash : bool_constant<!!sizeof(H)> {};
 template <>
 struct StdIsFastHash<std::hash<long double>> : std::false_type {};
 template <typename... Args>
@@ -225,7 +225,11 @@ class F14HashToken final {
   template <typename Policy>
   friend class f14::detail::F14Table;
 };
+#else
+class F14HashToken final {};
+#endif
 
+#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 namespace f14 {
 namespace detail {
 
@@ -247,7 +251,7 @@ FOLLY_ALWAYS_INLINE static void prefetchAddr(T const* ptr) {
   __builtin_prefetch(static_cast<void const*>(ptr));
 #elif FOLLY_NEON
   __prefetch(static_cast<void const*>(ptr));
-#else
+#elif FOLLY_SSE >= 2
   _mm_prefetch(
       static_cast<char const*>(static_cast<void const*>(ptr)), _MM_HINT_T0);
 #endif
@@ -255,8 +259,10 @@ FOLLY_ALWAYS_INLINE static void prefetchAddr(T const* ptr) {
 
 #if FOLLY_NEON
 using TagVector = uint8x16_t;
-#else // SSE2
+#elif FOLLY_SSE >= 2
 using TagVector = __m128i;
+#elif FOLLY_HAVE_INT128_T
+using TagVector = __uint128_t;
 #endif
 
 // We could use unaligned loads to relax this requirement, but that
@@ -269,7 +275,7 @@ using EmptyTagVectorType = std::aligned_storage_t<
     sizeof(TagVector) + kRequiredVectorAlignment,
     alignof(max_align_t)>;
 
-extern EmptyTagVectorType kEmptyTagVector;
+FOLLY_EXPORT extern EmptyTagVectorType kEmptyTagVector;
 
 template <typename ItemType>
 struct alignas(kRequiredVectorAlignment) F14Chunk {
@@ -324,7 +330,10 @@ struct alignas(kRequiredVectorAlignment) F14Chunk {
     }
     auto rv = reinterpret_cast<F14Chunk*>(raw);
     FOLLY_SAFE_DCHECK(
-        (reinterpret_cast<uintptr_t>(rv) % kRequiredVectorAlignment) == 0, "");
+        (reinterpret_cast<uintptr_t>(rv) % kRequiredVectorAlignment) == 0,
+        reinterpret_cast<uintptr_t>(rv),
+        " not aligned to ",
+        kRequiredVectorAlignment);
     return rv;
   }
 
@@ -433,7 +442,7 @@ struct alignas(kRequiredVectorAlignment) F14Chunk {
     uint8x8_t maskV = vshrn_n_u16(vreinterpretq_u16_u8(occupiedV), 4);
     return vget_lane_u64(vreinterpret_u64_u8(maskV), 0) & kFullMask;
   }
-#else
+#elif FOLLY_SSE >= 2
   ////////
   // Tag filtering using SSE2 intrinsics
 
@@ -471,6 +480,38 @@ struct alignas(kRequiredVectorAlignment) F14Chunk {
     auto tagV = _mm_load_si128(tagVector());
     return _mm_movemask_epi8(tagV) & kFullMask;
   }
+#elif FOLLY_HAVE_INT128_T
+  ////////
+  // Tag filtering using plain C/C++
+
+  SparseMaskIter tagMatchIter(std::size_t needle) const {
+    FOLLY_SAFE_DCHECK(needle >= 0x80 && needle < 0x100, "");
+    auto tagV = static_cast<uint8_t const*>(&tags_[0]);
+    MaskType mask = 0;
+#if defined(__GNUC__)
+#pragma GCC unroll 16
+#else
+#pragma unroll 16
+#endif
+    for (int i = 0; i < kCapacity; i++) {
+      mask |= ((tagV[i] == static_cast<uint8_t>(needle)) ? 1 : 0) << i;
+    }
+    return SparseMaskIter{mask & kFullMask};
+  }
+
+  MaskType occupiedMask() const {
+    auto tagV = static_cast<uint8_t const*>(&tags_[0]);
+    MaskType mask = 0;
+#if defined(__GNUC__)
+#pragma GCC unroll 16
+#else
+#pragma unroll 16
+#endif
+    for (int i = 0; i < kCapacity; i++) {
+      mask |= ((tagV[i] & 0x80) ? 1 : 0) << i;
+    }
+    return mask & kFullMask;
+  }
 #endif
 
   DenseMaskIter occupiedIter() const {
@@ -695,7 +736,7 @@ class F14ItemIter {
     while (index_ > 0) {
       --index_;
       --itemPtr_;
-      if (LIKELY(c->occupied(index_))) {
+      if (FOLLY_LIKELY(c->occupied(index_))) {
         return;
       }
     }
@@ -724,7 +765,7 @@ class F14ItemIter {
     for (std::size_t i = 1; !likelyDead || i != 0; ++i) {
       if (checkEof) {
         // exhausted the current chunk
-        if (UNLIKELY(c->eof())) {
+        if (FOLLY_UNLIKELY(c->eof())) {
           FOLLY_SAFE_DCHECK(index_ == 0, "");
           itemPtr_ = nullptr;
           return;
@@ -737,7 +778,7 @@ class F14ItemIter {
       if (checkEof && !likelyDead) {
         prefetchAddr(&*c - 1);
       }
-      if (LIKELY(last.hasIndex())) {
+      if (FOLLY_LIKELY(last.hasIndex())) {
         index_ = last.index();
         itemPtr_ = std::pointer_traits<ItemPtr>::pointer_to(c->item(index_));
         return;
@@ -1153,7 +1194,7 @@ class F14Table : public Policy {
   // For hash functions we don't trust to avalanche, we repair things by
   // applying a bit mixer to the user-supplied hash.
 
-#if FOLLY_X64 || FOLLY_AARCH64
+#if FOLLY_X64 || FOLLY_AARCH64 || FOLLY_RISCV64
   // 64-bit
   static HashPair splitHash(std::size_t hash) {
     static_assert(sizeof(std::size_t) == sizeof(uint64_t), "");
@@ -1448,14 +1489,14 @@ class F14Table : public Policy {
       auto hits = chunk->tagMatchIter(hp.second);
       while (hits.hasNext()) {
         auto i = hits.next();
-        if (LIKELY(this->keyMatchesItem(key, chunk->item(i)))) {
+        if (FOLLY_LIKELY(this->keyMatchesItem(key, chunk->item(i)))) {
           // Tag match and key match were both successful.  The chance
           // of a false tag match is 1/128 for each key in the chunk
           // (with a proper hash function).
           return ItemIter{chunk, i};
         }
       }
-      if (LIKELY(chunk->outboundOverflowCount() == 0)) {
+      if (FOLLY_LIKELY(chunk->outboundOverflowCount() == 0)) {
         // No keys that wanted to be placed in this chunk were denied
         // entry, so our search is over.  This is the common case.
         break;
@@ -1518,12 +1559,12 @@ class F14Table : public Policy {
       auto hits = chunk->tagMatchIter(hp.second);
       while (hits.hasNext()) {
         auto i = hits.next();
-        if (LIKELY(
+        if (FOLLY_LIKELY(
                 func(this->keyForValue(this->valueAtItem(chunk->item(i)))))) {
           return ItemIter{chunk, i};
         }
       }
-      if (LIKELY(chunk->outboundOverflowCount() == 0)) {
+      if (FOLLY_LIKELY(chunk->outboundOverflowCount() == 0)) {
         break;
       }
       index += step;
@@ -1600,7 +1641,7 @@ class F14Table : public Policy {
     while (true) {
       index = moduloByChunkCount(index);
       chunk = chunks_ + index;
-      if (LIKELY(fullness[index] < Chunk::kCapacity)) {
+      if (FOLLY_LIKELY(fullness[index] < Chunk::kCapacity)) {
         break;
       }
       chunk->incrOutboundOverflowCount();
@@ -1843,31 +1884,11 @@ class F14Table : public Policy {
     }
   }
 
-  void reserveImpl(std::size_t desiredCapacity) {
-    desiredCapacity = std::max<std::size_t>(desiredCapacity, size());
-    if (desiredCapacity == 0) {
-      reset();
-      return;
-    }
-
+  void maybeRehash(std::size_t desiredCapacity, bool attemptExact) {
     auto origChunkCount = chunkCount();
     auto origCapacityScale = chunks_->capacityScale();
     auto origCapacity = computeCapacity(origChunkCount, origCapacityScale);
 
-    // This came from an explicit reserve() or rehash() call, so there's
-    // a good chance the capacity is exactly right.  To avoid O(n^2)
-    // behavior, we don't do rehashes that decrease the size by less
-    // than 1/8, and if we have a requested increase of less than 1/8 we
-    // instead go to the next power of two.
-
-    if (desiredCapacity <= origCapacity &&
-        desiredCapacity >= origCapacity - origCapacity / 8) {
-      return;
-    }
-    bool attemptExact =
-        !(desiredCapacity > origCapacity &&
-          desiredCapacity < origCapacity + origCapacity / 8);
-
     std::size_t newChunkCount;
     std::size_t newCapacityScale;
     std::tie(newChunkCount, newCapacityScale) = computeChunkCountAndScale(
@@ -1884,6 +1905,33 @@ class F14Table : public Policy {
     }
   }
 
+  void reserveImpl(std::size_t requestedCapacity) {
+    const size_t targetCapacity =
+        std::max<std::size_t>(requestedCapacity, size());
+    if (targetCapacity == 0) {
+      reset();
+      return;
+    }
+
+    // Special case reserve(n) for n <= size() (pseudo "shrink_to_fit")
+    if (requestedCapacity <= size()) {
+      maybeRehash(targetCapacity, /*attemptExact*/ true);
+      return;
+    }
+
+    auto origCapacity = bucket_count();
+
+    // Never shrink in order to avoid O(n^2) behavior of repeated reserves
+    if (targetCapacity <= origCapacity) {
+      return;
+    }
+
+    // Large increase? Good chance the capacity is exactly right
+    bool attemptExact =
+        targetCapacity > origCapacity + ((origCapacity + 7) / 8);
+    maybeRehash(targetCapacity, attemptExact);
+  }
+
   FOLLY_NOINLINE void reserveForInsertImpl(
       std::size_t capacityMinusOne,
       std::size_t origChunkCount,
@@ -1967,7 +2015,7 @@ class F14Table : public Policy {
       // this SCOPE_EXIT reverts chunks_ and chunkShift if necessary
       BytePtr finishedRawAllocation = nullptr;
       std::size_t finishedAllocSize = 0;
-      if (LIKELY(success)) {
+      if (FOLLY_LIKELY(success)) {
         if (origCapacity > 0) {
           finishedRawAllocation = std::pointer_traits<BytePtr>::pointer_to(
               *static_cast<uint8_t*>(static_cast<void*>(&*origChunks)));
@@ -2000,7 +2048,7 @@ class F14Table : public Policy {
       std::size_t srcI = 0;
       std::size_t dstI = 0;
       while (dstI < origSize) {
-        if (LIKELY(srcChunk->occupied(srcI))) {
+        if (FOLLY_LIKELY(srcChunk->occupied(srcI))) {
           dstChunk->setTag(dstI, srcChunk->tag(srcI));
           this->moveItemDuringRehash(
               dstChunk->itemAddr(dstI), srcChunk->item(srcI));
@@ -2304,7 +2352,7 @@ class F14Table : public Policy {
 
   template <typename K, typename BeforeDestroy>
   std::size_t eraseKeyInto(K const& key, BeforeDestroy&& beforeDestroy) {
-    if (UNLIKELY(size() == 0)) {
+    if (FOLLY_UNLIKELY(size() == 0)) {
       return 0;
     }
     auto hp = splitHash(this->computeKeyHash(key));
