@@ -20,6 +20,7 @@
 #include <chrono>
 #include <random>
 #include <string>
+#include <type_traits>
 #include <typeinfo>
 #include <unordered_map>
 
@@ -28,6 +29,7 @@
 #include <folly/Benchmark.h>
 #include <folly/Conv.h>
 #include <folly/FBString.h>
+#include <folly/Portability.h>
 #include <folly/container/test/F14TestUtil.h>
 #include <folly/container/test/TrackingTypes.h>
 #include <folly/hash/Hash.h>
@@ -204,7 +206,7 @@ TEST(F14FastMap, visitContiguousRanges) {
 }
 
 #if FOLLY_HAS_MEMORY_RESOURCE
-TEST(F14Map, pmr_empty) {
+TEST(F14Map, pmrEmpty) {
   pmr::F14ValueMap<int, int> m1;
   pmr::F14NodeMap<int, int> m2;
   pmr::F14VectorMap<int, int> m3;
@@ -767,25 +769,25 @@ TEST(F14FastMap, simple) {
 }
 
 #if FOLLY_HAS_MEMORY_RESOURCE
-TEST(F14ValueMap, pmr_simple) {
+TEST(F14ValueMap, pmrSimple) {
   runSimple<pmr::F14ValueMap<std::string, std::string>>();
 }
 
-TEST(F14NodeMap, pmr_simple) {
+TEST(F14NodeMap, pmrSimple) {
   runSimple<pmr::F14NodeMap<std::string, std::string>>();
 }
 
-TEST(F14VectorMap, pmr_simple) {
+TEST(F14VectorMap, pmrSimple) {
   runSimple<pmr::F14VectorMap<std::string, std::string>>();
 }
 
-TEST(F14FastMap, pmr_simple) {
+TEST(F14FastMap, pmrSimple) {
   runSimple<pmr::F14FastMap<std::string, std::string>>();
 }
 #endif
 
 #if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
-TEST(F14VectorMap, reverse_iterator) {
+TEST(F14VectorMap, reverseIterator) {
   using TMap = F14VectorMap<uint64_t, uint64_t>;
   auto populate = [](TMap& h, uint64_t lo, uint64_t hi) {
     for (auto i = lo; i < hi; ++i) {
@@ -872,7 +874,6 @@ TEST(F14FastMap, rehash) {
   runRehash<F14VectorMap<std::string, std::string>>();
 }
 
-#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 template <typename T>
 void runPrehash() {
   T h;
@@ -906,7 +907,6 @@ TEST(F14VectorMap, prehash) {
 TEST(F14FastMap, prehash) {
   runPrehash<F14FastMap<std::string, std::string>>();
 }
-#endif
 
 TEST(F14ValueMap, random) {
   runRandom<F14ValueMap<
@@ -944,7 +944,7 @@ TEST(F14FastMap, random) {
       SwapTrackingAlloc<std::pair<uint64_t const, Tracked<1>>>>>();
 }
 
-TEST(F14ValueMap, grow_stats) {
+TEST(F14ValueMap, growStats) {
   F14ValueMap<uint64_t, uint64_t> h;
   for (unsigned i = 1; i <= 3072; ++i) {
     h[i]++;
@@ -956,7 +956,7 @@ TEST(F14ValueMap, grow_stats) {
   runSanityChecks(h);
 }
 
-TEST(F14ValueMap, steady_state_stats) {
+TEST(F14ValueMap, steadyStateStats) {
   // 10k keys, 14% probability of insert, 90% chance of erase, so the
   // table should converge to 1400 size without triggering the rehash
   // that would occur at 1536.
@@ -984,7 +984,7 @@ TEST(F14ValueMap, steady_state_stats) {
   runSanityChecks(h);
 }
 
-TEST(F14VectorMap, steady_state_stats) {
+TEST(F14VectorMap, steadyStateStats) {
   // 10k keys, 14% probability of insert, 90% chance of erase, so the
   // table should converge to 1400 size without triggering the rehash
   // that would occur at 1536.
@@ -1400,9 +1400,8 @@ TEST(F14VectorMap, destructuringErase) {
       0);
 }
 
+#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 TEST(F14ValueMap, maxSize) {
-  SKIP_IF(kFallback);
-
   F14ValueMap<int, int> m;
   EXPECT_EQ(
       m.max_size(),
@@ -1413,8 +1412,6 @@ TEST(F14ValueMap, maxSize) {
 }
 
 TEST(F14NodeMap, maxSize) {
-  SKIP_IF(kFallback);
-
   F14NodeMap<int, int> m;
   EXPECT_EQ(
       m.max_size(),
@@ -1425,8 +1422,6 @@ TEST(F14NodeMap, maxSize) {
 }
 
 TEST(F14VectorMap, vectorMaxSize) {
-  SKIP_IF(kFallback);
-
   F14VectorMap<int, int> m;
   EXPECT_EQ(
       m.max_size(),
@@ -1435,6 +1430,7 @@ TEST(F14VectorMap, vectorMaxSize) {
           std::allocator_traits<decltype(m)::allocator_type>::max_size(
               m.get_allocator())));
 }
+#endif
 
 template <typename M>
 void runMoveOnlyTest() {
@@ -1673,24 +1669,20 @@ TEST(F14ValueMap, heterogeneousLookup) {
     EXPECT_TRUE(ref.end() == ref.find(buddy));
     EXPECT_EQ(hello, ref.find(hello)->first);
 
-#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
     const auto buddyHashToken = ref.prehash(buddy);
     const auto helloHashToken = ref.prehash(hello);
 
     // prehash + find
     EXPECT_TRUE(ref.end() == ref.find(buddyHashToken, buddy));
     EXPECT_EQ(hello, ref.find(helloHashToken, hello)->first);
-#endif
 
     // contains
     EXPECT_FALSE(ref.contains(buddy));
     EXPECT_TRUE(ref.contains(hello));
 
-#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
     // contains with prehash
     EXPECT_FALSE(ref.contains(buddyHashToken, buddy));
     EXPECT_TRUE(ref.contains(helloHashToken, hello));
-#endif
 
     // equal_range
     EXPECT_TRUE(std::make_pair(ref.end(), ref.end()) == ref.equal_range(buddy));
@@ -2154,7 +2146,6 @@ TEST(F14Map, continuousCapacityF12) {
   runContinuousCapacityTest<F14VectorMap<uint16_t, uint16_t>>(0xfff0, 0xfffe);
 }
 
-#if FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
 template <template <class...> class TMap>
 void testContainsWithPrecomputedHash() {
   TMap<int, int> m{};
@@ -2176,7 +2167,6 @@ TEST(F14Map, containsWithPrecomputedHash) {
   testContainsWithPrecomputedHash<F14NodeMap>();
   testContainsWithPrecomputedHash<F14FastMap>();
 }
-#endif
 
 template <template <class...> class TMap>
 void testEraseIf() {
@@ -2234,3 +2224,275 @@ TEST(F14Map, copyAfterRemovedCollisions) {
   testCopyAfterRemovedCollisions<F14NodeMap>();
   testCopyAfterRemovedCollisions<F14FastMap>();
 }
+
+#if FOLLY_HAS_DEDUCTION_GUIDES && /* TODO: Implement deduction guides in \
+                                     fallback implementation. */         \
+    FOLLY_F14_VECTOR_INTRINSICS_AVAILABLE
+template <template <class...> class TMap>
+void testIterDeductionGuide() {
+  TMap<int, double> source({{1, 2.0}, {3, 4.0}});
+
+  TMap dest1(source.begin(), source.end());
+  static_assert(std::is_same_v<decltype(dest1), decltype(source)>);
+  EXPECT_EQ(dest1, source);
+
+  TMap dest2(source.begin(), source.end(), 2);
+  static_assert(std::is_same_v<decltype(dest2), decltype(source)>);
+  EXPECT_EQ(dest2, source);
+
+  TMap dest3(source.begin(), source.end(), 2, f14::DefaultHasher<int>{});
+  static_assert(std::is_same_v<decltype(dest3), decltype(source)>);
+  EXPECT_EQ(dest3, source);
+
+  TMap dest4(
+      source.begin(),
+      source.end(),
+      2,
+      f14::DefaultHasher<int>{},
+      f14::DefaultKeyEqual<int>{});
+  static_assert(std::is_same_v<decltype(dest4), decltype(source)>);
+  EXPECT_EQ(dest4, source);
+
+  TMap dest5(
+      source.begin(),
+      source.end(),
+      2,
+      f14::DefaultHasher<int>{},
+      f14::DefaultKeyEqual<int>{},
+      f14::DefaultAlloc<std::pair<const int, double>>{});
+  static_assert(std::is_same_v<decltype(dest5), decltype(source)>);
+  EXPECT_EQ(dest5, source);
+
+  TMap dest6(
+      source.begin(),
+      source.end(),
+      2,
+      f14::DefaultAlloc<std::pair<const int, double>>{});
+  static_assert(std::is_same_v<decltype(dest6), decltype(source)>);
+  EXPECT_EQ(dest6, source);
+
+  TMap dest7(
+      source.begin(),
+      source.end(),
+      2,
+      f14::DefaultHasher<int>{},
+      f14::DefaultAlloc<std::pair<const int, double>>{});
+  static_assert(std::is_same_v<decltype(dest7), decltype(source)>);
+  EXPECT_EQ(dest7, source);
+}
+
+TEST(F14Map, iterDeductionGuide) {
+  testIterDeductionGuide<F14ValueMap>();
+  testIterDeductionGuide<F14NodeMap>();
+  testIterDeductionGuide<F14VectorMap>();
+  testIterDeductionGuide<F14FastMap>();
+}
+
+template <template <class...> class TMap>
+void testInitializerListDeductionGuide() {
+  TMap<int, double> source({{1, 2.0}, {3, 4.0}});
+
+#if !defined(__GNUC__) || __GNUC__ > 12 || defined(__clang__)
+  // some versions of gcc, including until at least gcc v12, fail here
+  TMap dest1{std::pair{1, 2.0}, {3, 4.0}};
+  static_assert(std::is_same_v<decltype(dest1), decltype(source)>);
+  EXPECT_EQ(dest1, source);
+#endif
+
+  TMap dest2({std::pair{1, 2.0}, {3, 4.0}});
+  static_assert(std::is_same_v<decltype(dest2), decltype(source)>);
+  EXPECT_EQ(dest2, source);
+
+  TMap dest3({std::pair{1, 2.0}, {3, 4.0}}, 2);
+  static_assert(std::is_same_v<decltype(dest3), decltype(source)>);
+  EXPECT_EQ(dest3, source);
+
+  TMap dest4({std::pair{1, 2.0}, {3, 4.0}}, 2, f14::DefaultHasher<int>{});
+  static_assert(std::is_same_v<decltype(dest4), decltype(source)>);
+  EXPECT_EQ(dest4, source);
+
+  TMap dest5(
+      {std::pair{1, 2.0}, {3, 4.0}},
+      2,
+      f14::DefaultHasher<int>{},
+      f14::DefaultKeyEqual<int>{});
+  static_assert(std::is_same_v<decltype(dest5), decltype(source)>);
+  EXPECT_EQ(dest5, source);
+
+  TMap dest6(
+      {std::pair{1, 2.0}, {3, 4.0}},
+      2,
+      f14::DefaultHasher<int>{},
+      f14::DefaultKeyEqual<int>{},
+      f14::DefaultAlloc<std::pair<const int, double>>{});
+  static_assert(std::is_same_v<decltype(dest6), decltype(source)>);
+  EXPECT_EQ(dest6, source);
+
+  TMap dest7(
+      {std::pair{1, 2.0}, {3, 4.0}},
+      2,
+      f14::DefaultAlloc<std::pair<const int, double>>{});
+  static_assert(std::is_same_v<decltype(dest7), decltype(source)>);
+  EXPECT_EQ(dest7, source);
+
+  TMap dest8(
+      {std::pair{1, 2.0}, {3, 4.0}},
+      2,
+      f14::DefaultHasher<int>{},
+      f14::DefaultAlloc<std::pair<const int, double>>{});
+  static_assert(std::is_same_v<decltype(dest8), decltype(source)>);
+  EXPECT_EQ(dest8, source);
+}
+
+TEST(F14Map, initializerListDeductionGuide) {
+  testInitializerListDeductionGuide<F14ValueMap>();
+  testInitializerListDeductionGuide<F14NodeMap>();
+  testInitializerListDeductionGuide<F14VectorMap>();
+  testInitializerListDeductionGuide<F14FastMap>();
+}
+#endif // FOLLY_HAS_DEDUCTION_GUIDES
+
+namespace {
+
+struct TracedMovable {
+ public:
+  TracedMovable() = default;
+
+  TracedMovable& operator=(TracedMovable&& other) noexcept {
+    if (this != &other) {
+      n = std::exchange(other.n, 0);
+    }
+    return *this;
+  }
+
+  TracedMovable(TracedMovable&& other) noexcept {
+    n = std::exchange(other.n, 0);
+  }
+
+  int32_t n{10};
+};
+
+} // namespace
+
+template <template <class...> class TMap>
+void testInsertOrAssignUnchangedIfNoInsert() {
+  TMap<int32_t, TracedMovable> m;
+
+  m[0] = TracedMovable{};
+  EXPECT_EQ(m[0].n, 10);
+
+  m.insert_or_assign(0, TracedMovable{});
+  EXPECT_EQ(m[0].n, 10);
+}
+
+TEST(F14Map, insertOrAssignUnchangedIfNoInsert) {
+  testInsertOrAssignUnchangedIfNoInsert<F14ValueMap>();
+  testInsertOrAssignUnchangedIfNoInsert<F14NodeMap>();
+  testInsertOrAssignUnchangedIfNoInsert<F14VectorMap>();
+  testInsertOrAssignUnchangedIfNoInsert<F14FastMap>();
+}
+
+template <typename M>
+void runSimpleShrinkToFitTest(float expectedLoadFactor) {
+  using K = typename M::key_type;
+  for (int n = 1; n <= 1000; ++n) {
+    M m;
+    for (K k = 0; k < n; ++k) {
+      m[k];
+    }
+    m.reserve(0); // reserve(0) works like shrink_to_fit
+    EXPECT_GE(m.load_factor(), expectedLoadFactor);
+  }
+}
+
+TEST(F14Map, shrinkToFit) {
+  SKIP_IF(kFallback);
+  runSimpleShrinkToFitTest<F14NodeMap<int, int>>(0.5);
+  runSimpleShrinkToFitTest<F14ValueMap<int, int>>(0.5);
+  runSimpleShrinkToFitTest<F14VectorMap<int, int>>(0.875);
+}
+
+template <typename M>
+void runDefactoShrinkToFitTest(float expectedLoadFactor) {
+  for (int n = 1; n <= 1000; n += (n + 9) / 10) {
+    M m1;
+    for (int k = 0; k < n; ++k) {
+      m1[k];
+    }
+    for (int j = 0; j <= n; ++j) {
+      auto m2 = m1;
+      // any argument < size is a "defacto" shrink_to_fit
+      m2.reserve(m2.size() - j);
+      EXPECT_GE(m2.load_factor(), expectedLoadFactor);
+    }
+  }
+}
+
+TEST(F14Map, defactoShrinkToFit) {
+  SKIP_IF(kFallback);
+  runDefactoShrinkToFitTest<F14NodeMap<int, int>>(0.5);
+  runDefactoShrinkToFitTest<F14ValueMap<int, int>>(0.5);
+  runDefactoShrinkToFitTest<F14VectorMap<int, int>>(0.875);
+}
+
+template <typename M>
+void runInitialReserveTest(float expectedLoadFactor) {
+  auto initBucketsCtor = [](int initBuckets) { return M(initBuckets); };
+  auto defaultCtorAndReserve = [](int initBuckets) {
+    M m;
+    m.reserve(initBuckets);
+    return m;
+  };
+
+  auto fill = [](M& m, int n) {
+    using K = typename M::key_type;
+    auto initBuckets = m.bucket_count();
+    for (K k = 0; k < n; ++k) {
+      m[k];
+      EXPECT_EQ(m.bucket_count(), initBuckets);
+    }
+  };
+
+  using MakeFuncs = std::initializer_list<std::function<M(int)>>;
+  for (const auto& make : MakeFuncs{initBucketsCtor, defaultCtorAndReserve}) {
+    for (int n = 1; n <= 1000; ++n) {
+      auto m = make(n);
+      fill(m, n);
+      EXPECT_GE(m.load_factor(), expectedLoadFactor);
+    }
+  }
+}
+
+TEST(F14Map, initialReserve) {
+  SKIP_IF(kFallback);
+  runInitialReserveTest<F14NodeMap<int, int>>(0.5);
+  runInitialReserveTest<F14ValueMap<int, int>>(0.5);
+  runInitialReserveTest<F14VectorMap<int, int>>(0.875);
+}
+
+template <typename M>
+void runReserveMoreTest(int n) {
+  constexpr int kIters = 1000;
+  M m;
+  int k = 0;
+  for (int i = 0; i < kIters; ++i) {
+    auto bc = m.bucket_count();
+    m.reserve(m.size() + n);
+    EXPECT_GE(m.bucket_count(), bc); // should never shrink
+    for (int j = 0; j < n; ++j) {
+      bc = m.bucket_count();
+      m[k++];
+      EXPECT_EQ(m.bucket_count(), bc);
+    }
+  }
+}
+
+TEST(F14Map, reserveMoreNeverShrinks) {
+  SKIP_IF(kFallback);
+  runReserveMoreTest<F14NodeMap<int, int>>(1);
+  runReserveMoreTest<F14ValueMap<int, int>>(1);
+  runReserveMoreTest<F14VectorMap<int, int>>(1);
+  runReserveMoreTest<F14NodeMap<int, int>>(10);
+  runReserveMoreTest<F14ValueMap<int, int>>(10);
+  runReserveMoreTest<F14VectorMap<int, int>>(10);
+}
