@@ -28,6 +28,8 @@ FOLLY_GFLAGS_DEFINE_bool(
 
 namespace folly {
 
+namespace {
+
 using folly::detail::MemoryIdler;
 
 /* Class that will free jemalloc caches and madvise the stack away
@@ -78,24 +80,32 @@ class MemoryIdlerTimeout : public AsyncTimeout, public EventBase::LoopCallback {
   size_t num_{0};
 };
 
+} // namespace
+
+// IOThreadPoolExecutorBase
+EventBase* IOThreadPoolExecutor::getEventBase(
+    ThreadPoolExecutor::ThreadHandle* h) {
+  auto thread = dynamic_cast<IOThread*>(h);
+
+  if (thread) {
+    return thread->eventBase;
+  }
+
+  return nullptr;
+}
+
+// IOThreadPoolExecutor
 IOThreadPoolExecutor::IOThreadPoolExecutor(
     size_t numThreads,
     std::shared_ptr<ThreadFactory> threadFactory,
     EventBaseManager* ebm,
     Options options)
-    : ThreadPoolExecutor(
+    : IOThreadPoolExecutor(
           numThreads,
           FLAGS_dynamic_iothreadpoolexecutor ? 0 : numThreads,
-          std::move(threadFactory)),
-      isWaitForAll_(options.waitForAll),
-      nextThread_(0),
-      eventBaseManager_(ebm) {
-  setNumThreads(numThreads);
-  registerThreadPoolExecutor(this);
-  if (options.enableThreadIdCollection) {
-    threadIdCollector_ = std::make_unique<ThreadIdWorkerProvider>();
-  }
-}
+          std::move(threadFactory),
+          ebm,
+          std::move(options)) {}
 
 IOThreadPoolExecutor::IOThreadPoolExecutor(
     size_t maxThreads,
@@ -103,7 +113,8 @@ IOThreadPoolExecutor::IOThreadPoolExecutor(
     std::shared_ptr<ThreadFactory> threadFactory,
     EventBaseManager* ebm,
     Options options)
-    : ThreadPoolExecutor(maxThreads, minThreads, std::move(threadFactory)),
+    : IOThreadPoolExecutorBase(
+          maxThreads, minThreads, std::move(threadFactory)),
       isWaitForAll_(options.waitForAll),
       nextThread_(0),
       eventBaseManager_(ebm) {
@@ -126,7 +137,7 @@ void IOThreadPoolExecutor::add(Func func) {
 void IOThreadPoolExecutor::add(
     Func func, std::chrono::milliseconds expiration, Func expireCallback) {
   ensureActiveThreads();
-  SharedMutex::ReadHolder r{&threadListLock_};
+  std::shared_lock r{threadListLock_};
   if (threadList_.get().empty()) {
     throw std::runtime_error("No threads available");
   }
@@ -171,7 +182,7 @@ IOThreadPoolExecutor::pickThread() {
 
 EventBase* IOThreadPoolExecutor::getEventBase() {
   ensureActiveThreads();
-  SharedMutex::ReadHolder r{&threadListLock_};
+  std::shared_lock r{threadListLock_};
   if (threadList_.get().empty()) {
     throw std::runtime_error("No threads available");
   }
@@ -182,7 +193,7 @@ std::vector<Executor::KeepAlive<EventBase>>
 IOThreadPoolExecutor::getAllEventBases() {
   ensureMaxActiveThreads();
   std::vector<Executor::KeepAlive<EventBase>> evbs;
-  SharedMutex::ReadHolder r{&threadListLock_};
+  std::shared_lock r{threadListLock_};
   const auto& threads = threadList_.get();
   evbs.reserve(threads.size());
   for (const auto& thr : threads) {
@@ -191,28 +202,6 @@ IOThreadPoolExecutor::getAllEventBases() {
   return evbs;
 }
 
-EventBase* IOThreadPoolExecutor::getEventBase(
-    ThreadPoolExecutor::ThreadHandle* h) {
-  auto thread = dynamic_cast<IOThread*>(h);
-
-  if (thread) {
-    return thread->eventBase;
-  }
-
-  return nullptr;
-}
-
-std::mutex* IOThreadPoolExecutor::getEventBaseShutdownMutex(
-    ThreadPoolExecutor::ThreadHandle* h) {
-  auto thread = dynamic_cast<IOThread*>(h);
-
-  if (thread) {
-    return &thread->eventBaseShutdownMutex_;
-  }
-
-  return nullptr;
-}
-
 EventBaseManager* IOThreadPoolExecutor::getEventBaseManager() {
   return eventBaseManager_;
 }
@@ -276,6 +265,7 @@ void IOThreadPoolExecutor::stopThreads(size_t n) {
         std::static_pointer_cast<IOThread>(threadList_.get()[i]);
     for (auto& o : observers_) {
       o->threadStopped(ioThread.get());
+      handleObserverUnregisterThread(ioThread.get(), *o);
     }
     ioThread->shouldRun = false;
     stoppedThreads.push_back(ioThread);
@@ -304,4 +294,20 @@ size_t IOThreadPoolExecutor::getPendingTaskCountImpl() const {
   return count;
 }
 
+void IOThreadPoolExecutor::handleObserverRegisterThread(
+    ThreadHandle* h, Observer& observer) {
+  auto thread = CHECK_NOTNULL(dynamic_cast<IOThread*>(h));
+  if (auto ioObserver = dynamic_cast<IOObserver*>(&observer)) {
+    ioObserver->registerEventBase(*thread->eventBase);
+  }
+}
+
+void IOThreadPoolExecutor::handleObserverUnregisterThread(
+    ThreadHandle* h, Observer& observer) {
+  auto thread = CHECK_NOTNULL(dynamic_cast<IOThread*>(h));
+  if (auto ioObserver = dynamic_cast<IOObserver*>(&observer)) {
+    ioObserver->unregisterEventBase(*thread->eventBase);
+  }
+}
+
 } // namespace folly
