@@ -252,8 +252,7 @@ class BitVectorReader : detail::ForwardPointers<Encoder::forwardQuantum>,
         detail::SkipPointers<Encoder::skipQuantum>(list.skipPointers),
         bits_(list.bits),
         size_(list.size),
-        upperBound_(
-            (kUnchecked || UNLIKELY(list.size == 0)) ? 0 : list.upperBound) {
+        upperBound_(kUnchecked || list.size == 0 ? 0 : list.upperBound) {
     reset();
   }
 
@@ -266,7 +265,7 @@ class BitVectorReader : detail::ForwardPointers<Encoder::forwardQuantum>,
   }
 
   bool next() {
-    if (!kUnchecked && UNLIKELY(position() + 1 >= size_)) {
+    if (!kUnchecked && FOLLY_UNLIKELY(position() + 1 >= size_)) {
       return setDone();
     }
 
@@ -291,7 +290,7 @@ class BitVectorReader : detail::ForwardPointers<Encoder::forwardQuantum>,
       return setDone();
     }
     // Small skip optimization.
-    if (LIKELY(n < kLinearScanThreshold)) {
+    if (FOLLY_LIKELY(n < kLinearScanThreshold)) {
       for (size_t i = 0; i < n; ++i) {
         next();
       }
