@@ -30,16 +30,32 @@
 namespace __cxxabiv1 {
 
 extern "C" {
+#ifdef FOLLY_STATIC_LIBSTDCXX
+void __real___cxa_throw(
+    void* thrownException, std::type_info* type, void (*destructor)(void*))
+    __attribute__((__noreturn__));
+void* __real___cxa_begin_catch(void* excObj) throw();
+void __real___cxa_rethrow(void) __attribute__((__noreturn__));
+void __real___cxa_end_catch(void);
+#else
 void __cxa_throw(
     void* thrownException, std::type_info* type, void (*destructor)(void*))
     __attribute__((__noreturn__));
 void* __cxa_begin_catch(void* excObj) throw();
 void __cxa_rethrow(void) __attribute__((__noreturn__));
 void __cxa_end_catch(void);
+#endif
 }
 
 } // namespace __cxxabiv1
 
+#ifdef FOLLY_STATIC_LIBSTDCXX
+extern "C" {
+void __real__ZSt17rethrow_exceptionNSt15__exception_ptr13exception_ptrE(
+    std::exception_ptr ep);
+} // namespace std
+#endif
+
 using namespace folly::exception_tracer;
 
 namespace {
@@ -100,6 +116,41 @@ FOLLY_EXNTRACE_DECLARE_CALLBACK(RethrowException)
 
 namespace __cxxabiv1 {
 
+#ifdef FOLLY_STATIC_LIBSTDCXX
+extern "C" {
+
+__attribute__((__noreturn__)) void __wrap___cxa_throw(
+    void* thrownException, std::type_info* type, void (*destructor)(void*)) {
+  getCxaThrowCallbacks().invoke(thrownException, type, &destructor);
+  __real___cxa_throw(thrownException, type, destructor);
+  __builtin_unreachable(); // orig_cxa_throw never returns
+}
+
+__attribute__((__noreturn__)) void __wrap___cxa_rethrow() {
+  // __cxa_rethrow leaves the current exception on the caught stack,
+  // and __cxa_begin_catch recognizes that case.  We could do the same, but
+  // we'll implement something simpler (and slower): we pop the exception from
+  // the caught stack, and push it back onto the active stack; this way, our
+  // implementation of __cxa_begin_catch doesn't have to do anything special.
+  getCxaRethrowCallbacks().invoke();
+  __real___cxa_rethrow();
+  __builtin_unreachable(); // orig_cxa_rethrow never returns
+}
+
+void* __wrap___cxa_begin_catch(void* excObj) throw() {
+  // excObj is a pointer to the unwindHeader in __cxa_exception
+  getCxaBeginCatchCallbacks().invoke(excObj);
+  return __real___cxa_begin_catch(excObj);
+}
+
+void __wrap___cxa_end_catch() {
+  getCxaEndCatchCallbacks().invoke();
+  __real___cxa_end_catch();
+}
+}
+
+#else
+
 void __cxa_throw(
     void* thrownException, std::type_info* type, void (*destructor)(void*)) {
   static auto orig_cxa_throw =
@@ -137,9 +188,25 @@ void __cxa_end_catch() {
   getCxaEndCatchCallbacks().invoke();
   orig_cxa_end_catch();
 }
+#endif
 
 } // namespace __cxxabiv1
 
+#ifdef FOLLY_STATIC_LIBSTDCXX
+// Mangled name for std::rethrow_exception
+// TODO(tudorb): Dicey, as it relies on the fact that std::exception_ptr
+// is typedef'ed to a type in namespace __exception_ptr
+extern "C" {
+void __wrap__ZSt17rethrow_exceptionNSt15__exception_ptr13exception_ptrE(
+    std::exception_ptr ep) {
+  getRethrowExceptionCallbacks().invoke(ep);
+  __real__ZSt17rethrow_exceptionNSt15__exception_ptr13exception_ptrE(ep);
+  __builtin_unreachable(); // orig_rethrow_exception never returns
+}
+}
+
+#else
+
 namespace std {
 
 void rethrow_exception(std::exception_ptr ep) {
@@ -151,10 +218,11 @@ void rethrow_exception(std::exception_ptr ep) {
           RTLD_NEXT,
           "_ZSt17rethrow_exceptionNSt15__exception_ptr13exception_ptrE"));
   getRethrowExceptionCallbacks().invoke(ep);
-  orig_rethrow_exception(ep);
+  orig_rethrow_exception(std::move(ep));
   __builtin_unreachable(); // orig_rethrow_exception never returns
 }
 
 } // namespace std
+#endif
 
 #endif // defined(__GLIBCXX__)
