@@ -23,7 +23,7 @@
 #include <folly/memory/Malloc.h>
 #include <folly/portability/SysUio.h>
 
-#if defined(__linux__) && __has_include(<liburing.h>)
+#if FOLLY_HAS_LIBURING
 
 namespace fsp = folly::portability::sockets;
 
@@ -134,7 +134,7 @@ std::string AsyncIoUringSocket::toString(AsyncIoUringSocket::State s) {
       return "Error";
     case State::FastOpen:
       return "FastOpen";
-  };
+  }
   return to<std::string>("Unknown val=", (int)s);
 }
 
@@ -146,7 +146,7 @@ AsyncIoUringSocket::Options::defaultAllocateNoBufferPoolBuffer() {
 }
 
 AsyncIoUringSocket::ReadSqe::ReadSqe(AsyncIoUringSocket* parent)
-    : parent_(parent) {
+    : IoSqeBase(IoSqeBase::Type::Read), parent_(parent) {
   supportsMultishotRecv_ = parent->options_.multishotRecv &&
       parent->backend_->kernelSupportsRecvmsgMultishot();
 }
@@ -821,7 +821,8 @@ AsyncIoUringSocket::WriteSqe::WriteSqe(
     std::unique_ptr<IOBuf>&& buf,
     WriteFlags flags,
     bool zc)
-    : parent_(parent),
+    : IoSqeBase(IoSqeBase::Type::Write),
+      parent_(parent),
       callback_(callback),
       buf_(std::move(buf)),
       flags_(flags),
@@ -926,7 +927,7 @@ struct DetachFdState : AsyncReader::ReadCallback {
 
 struct CancelSqe : IoSqeBase {
   explicit CancelSqe(IoSqeBase* sqe, folly::Function<void()> fn = {})
-      : target_(sqe), fn_(std::move(fn)) {}
+      : IoSqeBase(IoSqeBase::Type::Cancel), target_(sqe), fn_(std::move(fn)) {}
   void processSubmit(struct io_uring_sqe* sqe) noexcept override {
     ::io_uring_prep_cancel(sqe, target_, 0);
   }
@@ -1187,7 +1188,9 @@ AsyncIoUringSocket::FastOpenSqe::FastOpenSqe(
     AsyncIoUringSocket* parent,
     SocketAddress const& addr,
     std::unique_ptr<WriteSqe> i)
-    : parent_(parent), initialWrite(std::move(i)) {
+    : IoSqeBase(IoSqeBase::Type::Open),
+      parent_(parent),
+      initialWrite(std::move(i)) {
   addrLen_ = addr.getAddress(&addrStorage);
 }
 
