@@ -36,7 +36,7 @@
 #include <sys/timerfd.h>
 #endif
 
-#if defined(__linux__) && __has_include(<liburing.h>)
+#if FOLLY_HAS_LIBURING
 
 extern "C" FOLLY_ATTR_WEAK void eb_poll_loop_pre_hook(uint64_t* call_time);
 extern "C" FOLLY_ATTR_WEAK void eb_poll_loop_post_hook(
@@ -1640,6 +1640,9 @@ void IoUringBackend::internalSubmit(IoSqeBase& ioSqe) noexcept {
   auto* sqe = getSqe();
   setSubmitting();
   ioSqe.internalSubmit(sqe);
+  if (ioSqe.type() == IoSqeBase::Type::Write) {
+    numSendEvents_++;
+  }
   doneSubmitting();
 }
 
@@ -1695,6 +1698,13 @@ size_t IoUringBackend::getActiveEvents(WaitForEventsMode waitForEvents) {
       submitBusyCheck(waitingToSubmit_, WaitForEventsMode::WAIT);
       int ret = ::io_uring_peek_cqe(&ioRing_, &cqe);
       return ret;
+    } else if (useReqBatching()) {
+      struct __kernel_timespec timeout;
+      timeout.tv_sec = 0;
+      timeout.tv_nsec = options_.timeout * 1000;
+      int ret = ::io_uring_wait_cqes(
+          &ioRing_, &cqe, options_.batchSize, &timeout, nullptr);
+      return ret;
     } else {
       int ret = ::io_uring_wait_cqe(&ioRing_, &cqe);
       return ret;
@@ -1766,6 +1776,10 @@ size_t IoUringBackend::getActiveEvents(WaitForEventsMode waitForEvents) {
     folly::terminate_with<std::runtime_error>("BADR");
   } else if (ret == -EAGAIN) {
     return 0;
+  } else if (ret == -ETIME) {
+    if (cqe == nullptr) {
+      return 0;
+    }
   } else if (ret < 0) {
     LOG(ERROR) << "wait_cqe error: " << ret;
     return 0;
@@ -1780,6 +1794,7 @@ unsigned int IoUringBackend::internalProcessCqe(
 
   unsigned int count_more = 0;
   unsigned int count = 0;
+  unsigned int count_send = 0;
 
   checkLogOverflow(&ioRing_);
   do {
@@ -1793,6 +1808,9 @@ unsigned int IoUringBackend::internalProcessCqe(
       IoSqeBase* sqe = reinterpret_cast<IoSqeBase*>(cqe->user_data);
       if (cqe->user_data) {
         count++;
+        if (sqe->type() == IoSqeBase::Type::Write) {
+          count_send++;
+        }
         if (FOLLY_UNLIKELY(mode == InternalProcessCqeMode::CANCEL_ALL)) {
           sqe->markCancelled();
         }
@@ -1825,6 +1843,7 @@ unsigned int IoUringBackend::internalProcessCqe(
     }
   } while (true);
   numInsertedEvents_ -= (count - count_more);
+  numSendEvents_ -= count_send;
   return count;
 }
 
@@ -1854,7 +1873,20 @@ int IoUringBackend::submitBusyCheck(
       if (options_.flags & Options::Flags::POLL_CQ) {
         res = ::io_uring_submit(&ioRing_);
       } else {
-        res = ::io_uring_submit_and_wait(&ioRing_, 1);
+        if (useReqBatching()) {
+          struct io_uring_cqe* cqe;
+          struct __kernel_timespec timeout;
+          timeout.tv_sec = 0;
+          timeout.tv_nsec = options_.timeout * 1000;
+          res = ::io_uring_submit_and_wait_timeout(
+              &ioRing_,
+              &cqe,
+              options_.batchSize + numSendEvents_,
+              &timeout,
+              nullptr);
+        } else {
+          res = ::io_uring_submit_and_wait(&ioRing_, 1);
+        }
         if (res >= 0) {
           // no more waiting
           waitForEvents = WaitForEventsMode::DONT_WAIT;
@@ -2211,4 +2243,4 @@ bool IoUringBackend::kernelSupportsSendZC() {
 
 } // namespace folly
 
-#endif // __has_include(<liburing.h>)
+#endif
