@@ -16,6 +16,7 @@
 
 #pragma once
 
+#include <folly/Synchronized.h>
 #include <folly/experimental/observer/detail/ObserverManager.h>
 
 namespace folly {
@@ -142,12 +143,12 @@ AtomicObserver<T>& AtomicObserver<T>::operator=(Observer<T> observer) {
 template <typename T>
 T AtomicObserver<T>::get() const {
   auto version = cachedVersion_.load(std::memory_order_acquire);
-  if (UNLIKELY(
+  if (FOLLY_UNLIKELY(
           observer_.needRefresh(version) ||
           observer_detail::ObserverManager::inManagerThread())) {
-    SharedMutex::WriteHolder guard{refreshLock_};
+    std::unique_lock guard{refreshLock_};
     version = cachedVersion_.load(std::memory_order_acquire);
-    if (LIKELY(
+    if (FOLLY_LIKELY(
             observer_.needRefresh(version) ||
             observer_detail::ObserverManager::inManagerThread())) {
       auto snapshot = *observer_;
@@ -188,46 +189,12 @@ ReadMostlyAtomicObserver<T>::ReadMostlyAtomicObserver(Observer<T> observer)
 
 template <typename T>
 T ReadMostlyAtomicObserver<T>::get() const {
-  if (UNLIKELY(observer_detail::ObserverManager::inManagerThread())) {
+  if (FOLLY_UNLIKELY(observer_detail::ObserverManager::inManagerThread())) {
     return **observer_;
   }
   return cachedValue_.load(std::memory_order_relaxed);
 }
 
-template <typename T>
-ReadMostlyTLObserver<T>::ReadMostlyTLObserver(Observer<T> observer)
-    : observer_(std::move(observer)) {
-  refresh();
-}
-
-template <typename T>
-ReadMostlyTLObserver<T>::ReadMostlyTLObserver(
-    const ReadMostlyTLObserver<T>& other)
-    : ReadMostlyTLObserver(other.observer_) {}
-
-template <typename T>
-ReadMostlySharedPtr<const T> ReadMostlyTLObserver<T>::getShared() const {
-  if (!observer_.needRefresh(localSnapshot_->version_) &&
-      !observer_detail::ObserverManager::inManagerThread()) {
-    if (auto data = localSnapshot_->data_.lock()) {
-      return data;
-    }
-  }
-  return refresh();
-}
-
-template <typename T>
-ReadMostlySharedPtr<const T> ReadMostlyTLObserver<T>::refresh() const {
-  auto snapshot = observer_.getSnapshot();
-  auto globalData = globalData_.lock();
-  if (globalVersion_.load() < snapshot.getVersion()) {
-    globalData->reset(snapshot.getShared());
-    globalVersion_ = snapshot.getVersion();
-  }
-  *localSnapshot_ = LocalSnapshot(*globalData, globalVersion_.load());
-  return globalData->getShared();
-}
-
 struct CallbackHandle::Context {
   Optional<Observer<folly::Unit>> observer;
   Synchronized<bool> canceled{false};
@@ -311,24 +278,5 @@ Observer<observer_detail::ResultOfUnwrapSharedPtr<F>> makeValueObserver(
       });
 }
 
-template <typename T, template <typename> class Atom>
-typename HazptrObserver<T, Atom>::DefaultSnapshot
-HazptrObserver<T, Atom>::getSnapshot() const {
-  if (UNLIKELY(observer_detail::ObserverManager::inManagerThread())) {
-    // Wait for updates
-    observer_.getSnapshot();
-  }
-  return DefaultSnapshot(state_, domain_);
-}
-
-template <typename T, template <typename> class Atom>
-typename HazptrObserver<T, Atom>::LocalSnapshot
-HazptrObserver<T, Atom>::getLocalSnapshot() const {
-  if (UNLIKELY(observer_detail::ObserverManager::inManagerThread())) {
-    // Wait for updates
-    observer_.getSnapshot();
-  }
-  return LocalSnapshot(state_, domain_);
-}
 } // namespace observer
 } // namespace folly
