@@ -37,7 +37,7 @@ BENCHMARK(settings_get_bench, iters) {
 }
 
 int main(int argc, char** argv) {
-  folly::init(&argc, &argv);
+  folly::Init init(&argc, &argv);
   folly::runBenchmarks();
 
   return 0;
