@@ -34,13 +34,12 @@
 #include <folly/lang/Exception.h>
 
 /**
- * Cursor class for fast iteration over IOBuf chains.
+ * IOBuf Cursors provide fast iteration over IOBuf chains.
  *
- * Cursor - Read-only access
- *
- * RWPrivateCursor - Read-write access, assumes private access to IOBuf chain
- * RWUnshareCursor - Read-write access, calls unshare on write (COW)
- * Appender        - Write access, assumes private access to IOBuf chain
+ * - Cursor          - Read-only access
+ * - RWPrivateCursor - Read-write access, assumes private access to IOBuf chain
+ * - RWUnshareCursor - Read-write access, calls unshare on write (COW)
+ * - Appender        - Write access, assumes private access to IOBuf chain
  *
  * Note that RW cursors write in the preallocated part of buffers (that is,
  * between the buffer's data() and tail()), while Appenders append to the end
@@ -48,12 +47,29 @@
  * automatically adjust the buffer pointers, so you may only use one
  * Appender with a buffer chain; for this reason, Appenders assume private
  * access to the buffer (you need to call unshare() yourself if necessary).
- **/
+ *
+ * @file Cursor.h
+ */
+
 namespace folly {
 namespace io {
 
+class Cursor;
+class ThinCursor;
+
 namespace detail {
 
+// This is very useful in development, but the size perturbation is currently
+// causing some previously undetected bugs in unrelated projects to manifest in
+// CI-breaking ways.
+// TODO(davidgoldblatt): Fix this.
+#define FOLLY_IO_CURSOR_BORROW_CHECKING 0
+#if FOLLY_IO_CURSOR_BORROW_CHECKING
+#define FOLLY_IO_CURSOR_BORROW_DCHECK DCHECK
+#else
+#define FOLLY_IO_CURSOR_BORROW_DCHECK(ignored)
+#endif
+
 template <class Derived, class BufType>
 class CursorBase {
   // Make all the templated classes friends for copy constructor.
@@ -61,6 +77,9 @@ class CursorBase {
   friend class CursorBase;
 
  public:
+  /**
+   * Construct a cursor wrapping an IOBuf.
+   */
   explicit CursorBase(BufType* buf) : crtBuf_(buf), buffer_(buf) {
     if (crtBuf_) {
       crtPos_ = crtBegin_ = crtBuf_->data();
@@ -68,6 +87,11 @@ class CursorBase {
     }
   }
 
+  /**
+   * Constuct a bounded cursor wrapping an IOBuf.
+   *
+   * @param len An upper bound on the number of bytes available to this cursor.
+   */
   CursorBase(BufType* buf, size_t len) : crtBuf_(buf), buffer_(buf) {
     if (crtBuf_) {
       crtPos_ = crtBegin_ = crtBuf_->data();
@@ -115,6 +139,8 @@ class CursorBase {
 
   /**
    * Reset cursor to point to a new buffer.
+   *
+   * @methodset Modifiers
    */
   void reset(BufType* buf) {
     crtBuf_ = buf;
@@ -129,12 +155,19 @@ class CursorBase {
 
   /**
    * Get the current Cursor position relative to the head of IOBuf chain.
+   *
+   * @methodset Capacity
    */
   size_t getCurrentPosition() const {
     dcheckIntegrity();
     return (crtPos_ - crtBegin_) + absolutePos_;
   }
 
+  /**
+   * Get the data at the current cursor position.
+   *
+   * @methodset Accessors
+   */
   const uint8_t* data() const {
     dcheckIntegrity();
     return crtPos_;
@@ -143,6 +176,8 @@ class CursorBase {
   /**
    * Return the remaining space available in the current IOBuf.
    *
+   * @methodset Capacity
+   *
    * May return 0 if the cursor is at the end of an IOBuf.  Use peekBytes()
    * instead if you want to avoid this.  peekBytes() will advance to the next
    * non-empty IOBuf (up to the end of the chain) if the cursor is currently
@@ -155,6 +190,9 @@ class CursorBase {
 
   /**
    * Return the space available until the end of the entire IOBuf chain.
+   *
+   * @methodset Capacity
+   *
    * For bounded Cursors, return the available space until the boundary.
    */
   size_t totalLength() const {
@@ -170,6 +208,9 @@ class CursorBase {
   /**
    * Return true if the cursor could advance the specified number of bytes
    * from its current position.
+   *
+   * @methodset Capacity
+   *
    * This is useful for applications that want to do checked reads instead of
    * catching exceptions and is more efficient than using totalLength as it
    * walks the minimal set of buffers in the chain to determine the result.
@@ -191,8 +232,10 @@ class CursorBase {
     return false;
   }
 
-  /*
+  /**
    * Return true if the cursor is at the end of the entire IOBuf chain.
+   *
+   * @methodset Capacity
    */
   bool isAtEnd() const {
     dcheckIntegrity();
@@ -221,6 +264,8 @@ class CursorBase {
 
   /**
    * Advances the cursor to the end of the entire IOBuf chain.
+   *
+   * @methodset Modifiers
    */
   void advanceToEnd() {
     // Simple case, we're already in the last IOBuf.
@@ -252,22 +297,34 @@ class CursorBase {
     }
   }
 
+  /// Advance the cursor
+  ///
+  /// @methodset Modifiers
   Derived& operator+=(size_t offset) {
     Derived* p = static_cast<Derived*>(this);
     p->skip(offset);
     return *p;
   }
+  /// Get a new cursor, advanced by offset from this cursor.
+  ///
+  /// @methodset Modifiers
   Derived operator+(size_t offset) const {
     Derived other(*this);
     other.skip(offset);
     return other;
   }
 
+  /// Retreat the cursor
+  ///
+  /// @methodset Modifiers
   Derived& operator-=(size_t offset) {
     Derived* p = static_cast<Derived*>(this);
     p->retreat(offset);
     return *p;
   }
+  /// Get a new cursor, retreated by offset from this cursor.
+  ///
+  /// @methodset Modifiers
   Derived operator-(size_t offset) const {
     Derived other(*this);
     other.retreat(offset);
@@ -277,10 +334,12 @@ class CursorBase {
   /**
    * Compare cursors for equality/inequality.
    *
+   * @methodset Comparison
+   *
    * Two cursors are equal if they are pointing to the same location in the
    * same IOBuf chain.
    */
-  bool operator==(const Derived& other) const {
+  bool operator==(const CursorBase& other) const {
     const IOBuf* crtBuf = crtBuf_;
     auto crtPos = crtPos_;
     // We can be pointing to the end of a buffer chunk, find first non-empty.
@@ -299,12 +358,24 @@ class CursorBase {
     }
     return (crtPos == crtPosOther) && (crtBuf == crtBufOther);
   }
-  bool operator!=(const Derived& other) const { return !operator==(other); }
 
+  /// @copydoc operator==
+  bool operator!=(const CursorBase& other) const { return !operator==(other); }
+
+  /**
+   * Attempt to read from the cursor.
+   *
+   * @methodset Consumers
+   *
+   * @param[out] val  Store the read value in this location.
+   * @return True iff successful; If there are not enough bytes left in the
+   * cursor, return false.
+   * @note val might be modified even if tryRead returns false.
+   */
   template <class T>
   typename std::enable_if<std::is_arithmetic<T>::value, bool>::type tryRead(
       T& val) {
-    if (LIKELY(uintptr_t(crtPos_) + sizeof(T) <= uintptr_t(crtEnd_))) {
+    if (FOLLY_LIKELY(uintptr_t(crtPos_) + sizeof(T) <= uintptr_t(crtEnd_))) {
       val = loadUnaligned<T>(data());
       crtPos_ += sizeof(T);
       return true;
@@ -312,6 +383,13 @@ class CursorBase {
     return pullAtMostSlow(&val, sizeof(T)) == sizeof(T);
   }
 
+  /**
+   * Attempt to read a Big-Endian integral from the cursor.
+   *
+   * @methodset Consumers
+   *
+   * @see tryRead
+   */
   template <class T>
   bool tryReadBE(T& val) {
     const bool result = tryRead(val);
@@ -319,6 +397,13 @@ class CursorBase {
     return result;
   }
 
+  /**
+   * Attempt to read a Little-Endian integral from the cursor.
+   *
+   * @methodset Consumers
+   *
+   * @see tryRead
+   */
   template <class T>
   bool tryReadLE(T& val) {
     const bool result = tryRead(val);
@@ -326,9 +411,20 @@ class CursorBase {
     return result;
   }
 
+  /**
+   * Read a value from the cursor.
+   *
+   * @methodset Consumers
+   *
+   * This function does not perform bounds checking: there must be enough bytes
+   * left in the cursor to populate T.
+   *
+   * This function only works with types that are bit-copyable: it calls memcpy
+   * to reinterpret bits from the IOBuf as T.
+   */
   template <class T>
   T read() {
-    if (LIKELY(uintptr_t(crtPos_) + sizeof(T) <= uintptr_t(crtEnd_))) {
+    if (FOLLY_LIKELY(uintptr_t(crtPos_) + sizeof(T) <= uintptr_t(crtEnd_))) {
       T val = loadUnaligned<T>(data());
       crtPos_ += sizeof(T);
       return val;
@@ -337,11 +433,25 @@ class CursorBase {
     }
   }
 
+  /**
+   * Read a Big-Endian integral from the cursor.
+   *
+   * @methodset Consumers
+   *
+   * @see read
+   */
   template <class T>
   T readBE() {
     return Endian::big(read<T>());
   }
 
+  /**
+   * Read a Little-Endian integral from the cursor.
+   *
+   * @methodset Consumers
+   *
+   * @see read
+   */
   template <class T>
   T readLE() {
     return Endian::little(read<T>());
@@ -350,6 +460,8 @@ class CursorBase {
   /**
    * Read a fixed-length string.
    *
+   * @methodset Consumers
+   *
    * The std::string-based APIs should probably be avoided unless you
    * ultimately want the data to live in an std::string. You're better off
    * using the pull() APIs to copy into a raw buffer otherwise.
@@ -357,7 +469,7 @@ class CursorBase {
   std::string readFixedString(size_t len) {
     std::string str;
     str.reserve(len);
-    if (LIKELY(length() >= len)) {
+    if (FOLLY_LIKELY(length() >= len)) {
       str.append(reinterpret_cast<const char*>(data()), len);
       crtPos_ += len;
     } else {
@@ -367,8 +479,11 @@ class CursorBase {
   }
 
   /**
-   * Read a string consisting of bytes until the given terminator character is
-   * seen. Raises an std::length_error if maxLength bytes have been processed
+   * Read a string of bytes until the given terminator character is seen.
+   *
+   * @methodset Consumers
+   *
+   * Raises an std::length_error if maxLength bytes have been processed
    * before the terminator is seen.
    *
    * See comments in readFixedString() about when it's appropriate to use this
@@ -378,8 +493,10 @@ class CursorBase {
       char termChar = '\0',
       size_t maxLength = std::numeric_limits<size_t>::max());
 
-  /*
-   * Read all bytes until the specified predicate returns true.
+  /**
+   * @overloadbrief Read bytes until the specified predicate returns true.
+   *
+   * @methodset Consumers
    *
    * The predicate will be called on each byte in turn, until it returns false
    * or until the end of the IOBuf chain is reached.
@@ -389,17 +506,17 @@ class CursorBase {
   template <typename Predicate>
   std::string readWhile(const Predicate& predicate);
 
-  /*
-   * Read all bytes until the specified predicate returns true.
-   *
-   * This is a more generic version of readWhile() takes an arbitrary Output
+  /**
+   * This is a more generic version of readWhile(). It takes an arbitrary Output
    * object, and calls Output::append() with each chunk of matching data.
    */
   template <typename Predicate, typename Output>
   void readWhile(const Predicate& predicate, Output& out);
 
-  /*
-   * Skip all bytes until the specified predicate returns true.
+  /**
+   * Skip bytes until the specified predicate returns true.
+   *
+   * @methodset Consumers
    *
    * The predicate will be called on each byte in turn, until it returns false
    * or until the end of the IOBuf chain is reached.
@@ -407,18 +524,30 @@ class CursorBase {
   template <typename Predicate>
   void skipWhile(const Predicate& predicate);
 
+  /**
+   * Advance the cursor by at most len bytes.
+   *
+   * @methodset Modifiers
+   */
   size_t skipAtMost(size_t len) {
     dcheckIntegrity();
-    if (LIKELY(uintptr_t(crtPos_) + len < uintptr_t(crtEnd_))) {
+    if (FOLLY_LIKELY(uintptr_t(crtPos_) + len < uintptr_t(crtEnd_))) {
       crtPos_ += len;
       return len;
     }
     return skipAtMostSlow(len);
   }
 
+  /**
+   * Advance the cursor by len bytes.
+   *
+   * @methodset Modifiers
+   *
+   * @throws out_of_range if there aren't enough bytes left in the cursor.
+   */
   void skip(size_t len) {
     dcheckIntegrity();
-    if (LIKELY(uintptr_t(crtPos_) + len < uintptr_t(crtEnd_))) {
+    if (FOLLY_LIKELY(uintptr_t(crtPos_) + len < uintptr_t(crtEnd_))) {
       crtPos_ += len;
     } else {
       skipSlow(len);
@@ -427,13 +556,21 @@ class CursorBase {
 
   /**
    * Skip bytes in the current IOBuf without advancing to the next one.
-   * Precondition: length() >= len
+   *
+   * @methodset Modifiers
+   *
+   * @pre length() >= len
    */
   void skipNoAdvance(size_t len) {
     DCHECK_LE(len, length());
     crtPos_ += len;
   }
 
+  /**
+   * Retreat the cursor by at most len bytes.
+   *
+   * @methodset Modifiers
+   */
   size_t retreatAtMost(size_t len) {
     dcheckIntegrity();
     if (len <= static_cast<size_t>(crtPos_ - crtBegin_)) {
@@ -443,6 +580,13 @@ class CursorBase {
     return retreatAtMostSlow(len);
   }
 
+  /**
+   * Retreat the cursor by at most len bytes.
+   *
+   * @methodset Modifiers
+   *
+   * @throws out_of_range if the cursor doesn't have enough bytes to retreat.
+   */
   void retreat(size_t len) {
     dcheckIntegrity();
     if (len <= static_cast<size_t>(crtPos_ - crtBegin_)) {
@@ -452,13 +596,23 @@ class CursorBase {
     }
   }
 
+  /**
+   * Copies at most len bytes from the cursor.
+   *
+   * @methodset Consumers
+   *
+   * The cursor will advance by the number of bytes read.
+   *
+   * @param[out] buf The buffer into which the bytes are copied.
+   * @returns The number of bytes copied.
+   */
   size_t pullAtMost(void* buf, size_t len) {
-    if (UNLIKELY(len == 0)) {
+    if (FOLLY_UNLIKELY(len == 0)) {
       return 0;
     }
     dcheckIntegrity();
     // Fast path: it all fits in one buffer.
-    if (LIKELY(uintptr_t(crtPos_) + len <= uintptr_t(crtEnd_))) {
+    if (FOLLY_LIKELY(uintptr_t(crtPos_) + len <= uintptr_t(crtEnd_))) {
       memcpy(buf, data(), len);
       crtPos_ += len;
       return len;
@@ -466,12 +620,22 @@ class CursorBase {
     return pullAtMostSlow(buf, len);
   }
 
+  /**
+   * Copies len bytes from the cursor.
+   *
+   * @methodset Consumers
+   *
+   * The cursor will advance by len.
+   *
+   * @param[out] buf The buffer into which the bytes are copied.
+   * @throw out_of_range if there aren't enough bytes in the cursor.
+   */
   void pull(void* buf, size_t len) {
-    if (UNLIKELY(len == 0)) {
+    if (FOLLY_UNLIKELY(len == 0)) {
       return;
     }
     dcheckIntegrity();
-    if (LIKELY(uintptr_t(crtPos_) + len <= uintptr_t(crtEnd_))) {
+    if (FOLLY_LIKELY(uintptr_t(crtPos_) + len <= uintptr_t(crtEnd_))) {
       memcpy(buf, data(), len);
       crtPos_ += len;
     } else {
@@ -480,14 +644,21 @@ class CursorBase {
   }
 
   /**
-   * Return the available data in the current buffer.
+   * Return the available data in the current IOBuf.
+   *
+   * @methodset Accessors
+   *
+   * Unlike data(), peekBytes() will advance to the next IOBuf while length()==0
+   * (though it can still return an empty range if there are no bytes left to
+   * read in the whole IOBuf chain).
+   *
    * If you want to gather more data from the chain into a contiguous region
    * (for hopefully zero-copy access), use gather() before peekBytes().
    */
   ByteRange peekBytes() {
     // Ensure that we're pointing to valid data
     size_t available = length();
-    while (UNLIKELY(available == 0 && tryAdvanceBuffer())) {
+    while (FOLLY_UNLIKELY(available == 0 && tryAdvanceBuffer())) {
       available = length();
     }
     return ByteRange{data(), available};
@@ -495,7 +666,9 @@ class CursorBase {
 
   /**
    * Alternate version of peekBytes() that returns a std::pair
-   * instead of a ByteRange.  (This method pre-dates ByteRange.)
+   * instead of a ByteRange.
+   *
+   * @methodset Accessors
    *
    * This function will eventually be deprecated.
    */
@@ -504,18 +677,34 @@ class CursorBase {
     return std::make_pair(bytes.data(), bytes.size());
   }
 
+  /**
+   * Clone len bytes from this cursor into an IOBuf.
+   *
+   * @methodset Accessors
+   *
+   * @param[out] buf The IOBuf into which to place the cloned data.
+   * @throws out_of_range if there aren't enough bytes in this cursor.
+   */
   void clone(std::unique_ptr<folly::IOBuf>& buf, size_t len) {
-    if (UNLIKELY(cloneAtMost(buf, len) != len)) {
+    if (FOLLY_UNLIKELY(cloneAtMost(buf, len) != len)) {
       throw_exception<std::out_of_range>("underflow");
     }
   }
 
   void clone(folly::IOBuf& buf, size_t len) {
-    if (UNLIKELY(cloneAtMost(buf, len) != len)) {
+    if (FOLLY_UNLIKELY(cloneAtMost(buf, len) != len)) {
       throw_exception<std::out_of_range>("underflow");
     }
   }
 
+  /**
+   * Clone at most len bytes from this cursor into an IOBuf.
+   *
+   * @methodset Accessors
+   *
+   * @param[out] buf The IOBuf into which to place the cloned data.
+   * @return The number of bytes actually cloned.
+   */
   size_t cloneAtMost(folly::IOBuf& buf, size_t len) {
     // We might be at the end of buffer.
     advanceBufferIfEmpty();
@@ -525,7 +714,7 @@ class CursorBase {
     for (int loopCount = 0; true; ++loopCount) {
       // Fast path: it all fits in one buffer.
       size_t available = length();
-      if (LIKELY(available >= len)) {
+      if (FOLLY_LIKELY(available >= len)) {
         if (loopCount == 0) {
           crtBuf_->cloneOneInto(buf);
           buf.trimStart(crtPos_ - crtBegin_);
@@ -552,7 +741,7 @@ class CursorBase {
       }
 
       copied += available;
-      if (UNLIKELY(!tryAdvanceBuffer())) {
+      if (FOLLY_UNLIKELY(!tryAdvanceBuffer())) {
         return copied;
       }
       len -= available;
@@ -621,12 +810,18 @@ class CursorBase {
     return len;
   }
 
+  /**
+   * Check if this cursor has a size limit imposed on it.
+   *
+   * @methodset Configuration
+   */
   bool isBounded() const {
     return remainingLen_ != std::numeric_limits<size_t>::max();
   }
 
  protected:
   void dcheckIntegrity() const {
+    FOLLY_IO_CURSOR_BORROW_DCHECK(!*borrowed());
     DCHECK(crtBegin_ <= crtPos_ && crtPos_ <= crtEnd_);
     DCHECK(crtBuf_ == nullptr || crtBegin_ == crtBuf_->data());
     DCHECK(
@@ -640,7 +835,7 @@ class CursorBase {
 
   bool tryAdvanceBuffer() {
     BufType* nextBuf = crtBuf_->next();
-    if (UNLIKELY(nextBuf == buffer_) || remainingLen_ == 0) {
+    if (FOLLY_UNLIKELY(nextBuf == buffer_) || remainingLen_ == 0) {
       crtPos_ = crtEnd_;
       return false;
     }
@@ -660,7 +855,7 @@ class CursorBase {
   }
 
   bool tryRetreatBuffer() {
-    if (UNLIKELY(crtBuf_ == buffer_)) {
+    if (FOLLY_UNLIKELY(crtBuf_ == buffer_)) {
       crtPos_ = crtBegin_;
       return false;
     }
@@ -693,6 +888,15 @@ class CursorBase {
   // is set to the max of size_t
   size_t remainingLen_{std::numeric_limits<size_t>::max()};
 
+#if FOLLY_IO_CURSOR_BORROW_CHECKING
+  bool borrowed_ = false;
+  bool* borrowed() { return &borrowed_; }
+  const bool* borrowed() const { return &borrowed_; }
+#else
+  bool* borrowed() { return nullptr; }
+  const bool* borrowed() const { return nullptr; }
+#endif
+
  private:
   Derived& derived() { return static_cast<Derived&>(*this); }
 
@@ -708,7 +912,7 @@ class CursorBase {
   FOLLY_NOINLINE void readFixedStringSlow(std::string* str, size_t len) {
     for (size_t available; (available = length()) < len;) {
       str->append(reinterpret_cast<const char*>(data()), available);
-      if (UNLIKELY(!tryAdvanceBuffer())) {
+      if (FOLLY_UNLIKELY(!tryAdvanceBuffer())) {
         throw_exception<std::out_of_range>("string underflow");
       }
       len -= available;
@@ -728,7 +932,7 @@ class CursorBase {
         memcpy(p, data(), available);
         copied += available;
       }
-      if (UNLIKELY(!tryAdvanceBuffer())) {
+      if (FOLLY_UNLIKELY(!tryAdvanceBuffer())) {
         return copied;
       }
       p += available;
@@ -745,7 +949,7 @@ class CursorBase {
   }
 
   FOLLY_NOINLINE void pullSlow(void* buf, size_t len) {
-    if (UNLIKELY(pullAtMostSlow(buf, len) != len)) {
+    if (FOLLY_UNLIKELY(pullAtMostSlow(buf, len) != len)) {
       throw_exception<std::out_of_range>("underflow");
     }
   }
@@ -754,7 +958,7 @@ class CursorBase {
     size_t skipped = 0;
     for (size_t available; (available = length()) < len;) {
       skipped += available;
-      if (UNLIKELY(!tryAdvanceBuffer())) {
+      if (FOLLY_UNLIKELY(!tryAdvanceBuffer())) {
         return skipped;
       }
       len -= available;
@@ -765,7 +969,7 @@ class CursorBase {
   }
 
   FOLLY_NOINLINE void skipSlow(size_t len) {
-    if (UNLIKELY(skipAtMostSlow(len) != len)) {
+    if (FOLLY_UNLIKELY(skipAtMostSlow(len) != len)) {
       throw_exception<std::out_of_range>("underflow");
     }
   }
@@ -774,7 +978,7 @@ class CursorBase {
     size_t retreated = 0;
     for (size_t available; (available = crtPos_ - crtBegin_) < len;) {
       retreated += available;
-      if (UNLIKELY(!tryRetreatBuffer())) {
+      if (FOLLY_UNLIKELY(!tryRetreatBuffer())) {
         return retreated;
       }
       len -= available;
@@ -784,7 +988,7 @@ class CursorBase {
   }
 
   FOLLY_NOINLINE void retreatSlow(size_t len) {
-    if (UNLIKELY(retreatAtMostSlow(len) != len)) {
+    if (FOLLY_UNLIKELY(retreatAtMostSlow(len) != len)) {
       throw_exception<std::out_of_range>("underflow");
     }
   }
@@ -792,8 +996,87 @@ class CursorBase {
   void advanceDone() {}
 };
 
+template <typename T>
+ThinCursor thinCursorReadSlow(ThinCursor, T&, Cursor&);
+ThinCursor thinCursorSkipSlow(ThinCursor, Cursor&, size_t);
 } // namespace detail
 
+// A register-pass facade for a Cursor. It strips out state that's only needed
+// down slow paths, so that it can be passed and returned efficiently in
+// registers. This gives up some convenience (users need to maintain that state
+// in a fallback Cursor), to gain performance.
+class ThinCursor {
+ public:
+  ThinCursor() = default;
+  ThinCursor(ThinCursor&&) = default;
+  ThinCursor(const ThinCursor&) = delete;
+  ThinCursor& operator=(ThinCursor&&) = default;
+  ThinCursor& operator=(const ThinCursor&) = delete;
+
+  const uint8_t* data() const {
+    dcheckIntegrity();
+    return crtPos_;
+  }
+
+  size_t length() const {
+    dcheckIntegrity();
+    return crtEnd_ - crtPos_;
+  }
+
+  bool canAdvance(size_t amount) const { return amount <= length(); }
+
+  bool isAtEnd() const { return length() == 0; }
+
+  template <class T>
+  FOLLY_ALWAYS_INLINE T read(Cursor& fallback) {
+    if (FOLLY_LIKELY((uintptr_t)crtEnd_ - (uintptr_t)crtPos_ >= sizeof(T))) {
+      T result = loadUnaligned<T>(data());
+      crtPos_ += sizeof(T);
+      return result;
+    } else {
+      T result;
+      *this = detail::thinCursorReadSlow(std::move(*this), result, fallback);
+      return result;
+    }
+  }
+
+  template <class T>
+  T readBE(Cursor& fallback) {
+    return Endian::big(read<T>(fallback));
+  }
+
+  template <class T>
+  T readLE(Cursor& fallback) {
+    return Endian::little(read<T>(fallback));
+  }
+
+  void skip(Cursor& fallback, size_t len) {
+    dcheckIntegrity();
+    if (FOLLY_LIKELY(uintptr_t(crtPos_) + len < uintptr_t(crtEnd_))) {
+      crtPos_ += len;
+    } else {
+      *this = detail::thinCursorSkipSlow(std::move(*this), fallback, len);
+    }
+  }
+
+  void skipNoAdvance(size_t len) {
+    DCHECK_LE(len, length());
+    crtPos_ += len;
+  }
+
+ private:
+  void dcheckIntegrity() const { DCHECK(crtPos_ <= crtEnd_); }
+
+  friend class Cursor;
+  ThinCursor(const uint8_t* crtPos, const uint8_t* crtEnd)
+      : crtPos_(crtPos), crtEnd_(crtEnd) {}
+  // Note: these are the only fields we can have -- x86-64 calling convention
+  // maxes out at returning 2 pointer-sized fields in registers, and we don't
+  // want to have to use memory for returning these.
+  const uint8_t* crtPos_;
+  const uint8_t* crtEnd_;
+};
+
 class Cursor : public detail::CursorBase<Cursor, const IOBuf> {
  public:
   explicit Cursor(const IOBuf* buf)
@@ -809,13 +1092,45 @@ class Cursor : public detail::CursorBase<Cursor, const IOBuf> {
   template <class OtherDerived, class OtherBuf>
   Cursor(const detail::CursorBase<OtherDerived, OtherBuf>& cursor, size_t len)
       : detail::CursorBase<Cursor, const IOBuf>(cursor, len) {}
+
+  ThinCursor borrow() {
+    FOLLY_IO_CURSOR_BORROW_DCHECK(!std::exchange(*borrowed(), true));
+    return {crtPos_, crtEnd_};
+  }
+
+  void unborrow(ThinCursor&& cursor) {
+    FOLLY_IO_CURSOR_BORROW_DCHECK(std::exchange(*borrowed(), false));
+    DCHECK_EQ(cursor.crtEnd_, crtEnd_);
+    crtPos_ = cursor.crtPos_;
+  }
 };
 
 namespace detail {
+template <typename T>
+ThinCursor thinCursorReadSlow(ThinCursor borrowed, T& val, Cursor& fallback) {
+  fallback.unborrow(std::move(borrowed));
+  val = fallback.read<T>();
+  return fallback.borrow();
+}
+
+inline ThinCursor thinCursorSkipSlow(
+    ThinCursor borrowed, Cursor& fallback, size_t len) {
+  fallback.unborrow(std::move(borrowed));
+  fallback.skip(len);
+  return fallback.borrow();
+}
 
 template <class Derived>
 class Writable {
  public:
+  /**
+   * Write a value to the cursor.
+   *
+   * @methodset Writing
+   *
+   * May throw if there isn't enough space and the derived cursor type does not
+   * support extending the IOBuf's writable range.
+   */
   template <class T>
   typename std::enable_if<std::is_arithmetic<T>::value>::type write(
       T value, size_t n = sizeof(T)) {
@@ -825,18 +1140,41 @@ class Writable {
     d->push(u8, n);
   }
 
+  /**
+   * Write a value to the cursor in Big-Endian.
+   *
+   * @methodset Writing
+   *
+   * May throw if there isn't enough space and the derived cursor type does not
+   * support extending the IOBuf's writable range.
+   */
   template <class T>
   void writeBE(T value) {
     Derived* d = static_cast<Derived*>(this);
     d->write(Endian::big(value));
   }
 
+  /**
+   * Write a value to the cursor in Little-Endian.
+   *
+   * @methodset Writing
+   *
+   * May throw if there isn't enough space and the derived cursor type does not
+   * support extending the IOBuf's writable range.
+   */
   template <class T>
   void writeLE(T value) {
     Derived* d = static_cast<Derived*>(this);
     d->write(Endian::little(value));
   }
 
+  /**
+   * Write bytes to the cursor.
+   *
+   * @methodset Writing
+   *
+   * @throw out_of_range if there isn't enough space in the cursor.
+   */
   void push(const uint8_t* buf, size_t len) {
     Derived* d = static_cast<Derived*>(this);
     if (d->pushAtMost(buf, len) != len) {
@@ -850,16 +1188,17 @@ class Writable {
     }
   }
 
+  /**
+   * Write bytes to the cursor; stop writing if the end of the cursor is
+   * reached.
+   *
+   * @methodset Writing
+   */
   size_t pushAtMost(ByteRange buf) {
     Derived* d = static_cast<Derived*>(this);
     return d->pushAtMost(buf.data(), buf.size());
   }
 
-  /**
-   * push len bytes of data from input cursor, data could be in an IOBuf chain.
-   * If input cursor contains less than len bytes, or this cursor has less than
-   * len bytes writable space, an out_of_range exception will be thrown.
-   */
   void push(Cursor cursor, size_t len) {
     if (this->pushAtMost(cursor, len) != len) {
       throw_exception<std::out_of_range>("overflow");
@@ -911,6 +1250,9 @@ class RWCursor : public detail::CursorBase<RWCursor<access>, IOBuf>,
   // Efficient way to advance to position cursor to the end of the queue,
   // using cached length instead of a walk via advanceToEnd().
   struct AtEnd {};
+  /**
+   * Create the cursor initially pointing to the end of queue.
+   */
   RWCursor(IOBufQueue& queue, AtEnd) : RWCursor(queue) {
     if (!queue.options().cacheChainLength) {
       this->advanceToEnd();
@@ -935,6 +1277,10 @@ class RWCursor : public detail::CursorBase<RWCursor<access>, IOBuf>,
   /**
    * Gather at least n bytes contiguously into the current buffer,
    * by coalescing subsequent buffers from the chain as necessary.
+   *
+   * @methodset Modifiers
+   *
+   * @throw overflow_error if there aren't enough bytes to gather
    */
   void gather(size_t n) {
     // Forbid attempts to gather beyond the end of this IOBuf chain.
@@ -954,6 +1300,13 @@ class RWCursor : public detail::CursorBase<RWCursor<access>, IOBuf>,
     this->crtEnd_ = this->crtBuf_->tail();
     this->crtPos_ = this->crtBegin_ + offset;
   }
+
+  /**
+   * Gather at most n bytes contiguously into the current buffer,
+   * by coalescing subsequent buffers from the chain as necessary.
+   *
+   * @methodset Modifiers
+   */
   void gatherAtMost(size_t n) {
     this->dcheckIntegrity();
     size_t size = std::min(n, this->totalLength());
@@ -978,7 +1331,7 @@ class RWCursor : public detail::CursorBase<RWCursor<access>, IOBuf>,
     for (;;) {
       // Fast path: the current buffer is big enough.
       size_t available = this->length();
-      if (LIKELY(available >= len)) {
+      if (FOLLY_LIKELY(available >= len)) {
         if (access == CursorAccess::UNSHARE) {
           maybeUnshare();
         }
@@ -992,7 +1345,7 @@ class RWCursor : public detail::CursorBase<RWCursor<access>, IOBuf>,
       }
       memcpy(writableData(), buf, available);
       copied += available;
-      if (UNLIKELY(!this->tryAdvanceBuffer())) {
+      if (FOLLY_UNLIKELY(!this->tryAdvanceBuffer())) {
         return copied;
       }
       buf += available;
@@ -1000,6 +1353,17 @@ class RWCursor : public detail::CursorBase<RWCursor<access>, IOBuf>,
     }
   }
 
+  /**
+   * Insert data at the cursor position.
+   *
+   * @methodset Writing
+   *
+   * Data in the IOBuf after the cursor will not be overwritten, though it might
+   * be moved.
+   *
+   * After this operator, the cursor will point to the data just after the
+   * inserted data.
+   */
   void insert(std::unique_ptr<folly::IOBuf> buf) {
     this->dcheckIntegrity();
     this->absolutePos_ += buf->computeChainDataLength();
@@ -1039,6 +1403,11 @@ class RWCursor : public detail::CursorBase<RWCursor<access>, IOBuf>,
     }
   }
 
+  /**
+   * Get a raw pointer to the writable section controlled by this cursor.
+   *
+   * @methodset Accessors
+   */
   uint8_t* writableData() {
     this->dcheckIntegrity();
     return this->crtBuf_->writableData() + (this->crtPos_ - this->crtBegin_);
@@ -1046,7 +1415,7 @@ class RWCursor : public detail::CursorBase<RWCursor<access>, IOBuf>,
 
  private:
   void maybeUnshare() {
-    if (UNLIKELY(maybeShared_)) {
+    if (FOLLY_UNLIKELY(maybeShared_)) {
       size_t offset = this->crtPos_ - this->crtBegin_;
       this->crtBuf_->unshareOne();
       this->crtBegin_ = this->crtBuf_->data();
@@ -1077,22 +1446,36 @@ class Appender : public detail::Writable<Appender> {
   Appender(IOBuf* buf, std::size_t growth)
       : buffer_(buf), crtBuf_(buf->prev()), growth_(growth) {}
 
+  /**
+   * Get the writable tail of the IOBuf this cursor points to.
+   *
+   * @methodset Accessors
+   */
   uint8_t* writableData() { return crtBuf_->writableTail(); }
 
+  /**
+   * Get the amount of writable tailroom of the IOBuf this cursor points to.
+   *
+   * @methodset Capacity
+   */
   size_t length() const { return crtBuf_->tailroom(); }
 
   /**
    * Mark n bytes (must be <= length()) as appended, as per the
    * IOBuf::append() method.
+   *
+   * @methodset Appending
    */
   void append(size_t n) { crtBuf_->append(n); }
 
   /**
    * Ensure at least n contiguous bytes available to write.
    * Postcondition: length() >= n.
+   *
+   * @methodset Appending
    */
   void ensure(std::size_t n) {
-    if (LIKELY(length() >= n)) {
+    if (FOLLY_LIKELY(length() >= n)) {
       return;
     }
 
@@ -1120,7 +1503,7 @@ class Appender : public detail::Writable<Appender> {
     // If the length of this buffer is 0 try growing it.
     // Otherwise on the first iteration of the following loop memcpy is called
     // with a null source pointer.
-    if (UNLIKELY(length() == 0 && !tryGrowChain())) {
+    if (FOLLY_UNLIKELY(length() == 0 && !tryGrowChain())) {
       return 0;
     }
 
@@ -1128,7 +1511,7 @@ class Appender : public detail::Writable<Appender> {
     for (;;) {
       // Fast path: it all fits in one buffer.
       size_t available = length();
-      if (LIKELY(available >= len)) {
+      if (FOLLY_LIKELY(available >= len)) {
         memcpy(writableData(), buf, len);
         append(len);
         return copied + len;
@@ -1137,7 +1520,7 @@ class Appender : public detail::Writable<Appender> {
       memcpy(writableData(), buf, available);
       append(available);
       copied += available;
-      if (UNLIKELY(!tryGrowChain())) {
+      if (FOLLY_UNLIKELY(!tryGrowChain())) {
         return copied;
       }
       buf += available;
@@ -1145,10 +1528,12 @@ class Appender : public detail::Writable<Appender> {
     }
   }
 
-  /*
+  /**
    * Append to the end of this buffer, using a printf() style
    * format specifier.
    *
+   * @methodset Appending
+   *
    * Note that folly/Format.h provides nicer and more type-safe mechanisms
    * for formatting strings, which should generally be preferred over
    * printf-style formatting.  Appender objects can be used directly as an
@@ -1170,12 +1555,15 @@ class Appender : public detail::Writable<Appender> {
   void printf(FOLLY_PRINTF_FORMAT const char* fmt, ...)
       FOLLY_PRINTF_FORMAT_ATTR(2, 3);
 
+  /// @methodset Appending
   void vprintf(const char* fmt, va_list ap);
 
-  /*
-   * Calling an Appender object with a StringPiece will append the string
-   * piece.  This allows Appender objects to be used directly with
-   * Formatter.
+  /**
+   * Append a StringPiece to the buffer.
+   *
+   * @methodset Appending
+   *
+   * This allows Appender objects to be used directly with Formatter.
    */
   void operator()(StringPiece sp) { push(ByteRange(sp)); }
 
@@ -1206,25 +1594,55 @@ class QueueAppender : public detail::Writable<QueueAppender> {
   QueueAppender(IOBufQueue* queue, std::size_t growth)
       : queueCache_(queue), growth_(growth) {}
 
+  /**
+   * Resets this, as if constructed anew.
+   */
   void reset(IOBufQueue* queue, std::size_t growth) {
     queueCache_.reset(queue);
     growth_ = growth;
   }
 
+  /**
+   * Get a pointer to the writable tail.
+   *
+   * @methodset Accessors
+   */
   uint8_t* writableData() { return queueCache_.writableData(); }
 
+  /**
+   * Get the size of the writable tail.
+   *
+   * @methodset Capacity
+   */
   size_t length() { return queueCache_.length(); }
 
+  /**
+   * Append n bytes.
+   *
+   * @methodset Appending
+   */
   void append(size_t n) { queueCache_.append(n); }
 
-  // Ensure at least n contiguous; can go above growth_, throws if
-  // not enough room.
+  /**
+   * Ensure that there are at least n contiguous bytes available for writing.
+   *
+   * @methodset Modifiers
+   *
+   * Can go above growth.
+   *
+   * May throw if there isn't enough room.
+   */
   void ensure(size_t n) {
     if (length() < n) {
       ensureSlow(n);
     }
   }
 
+  /**
+   * Write an object to the cursor.
+   *
+   * @param n The number of bytes of value to write; defaults to sizeof(T)
+   */
   template <class T>
   typename std::enable_if<std::is_arithmetic<T>::value>::type write(
       T value, size_t n = sizeof(T)) {
@@ -1260,6 +1678,11 @@ class QueueAppender : public detail::Writable<QueueAppender> {
     return len;
   }
 
+  /**
+   * Inserts data at the current cursor position.
+   *
+   * @methodset Writing
+   */
   void insert(std::unique_ptr<folly::IOBuf> buf) {
     if (buf) {
       queueCache_.queue()->append(
@@ -1272,11 +1695,21 @@ class QueueAppender : public detail::Writable<QueueAppender> {
         buf, /* pack */ true, /* allowTailReuse */ true);
   }
 
+  /**
+   * Get a RWCursor for this IOBufQueue.
+   *
+   * @methodset Accessors
+   */
   template <CursorAccess access>
   explicit operator RWCursor<access>() {
     return RWCursor<access>(*queueCache_.queue());
   }
 
+  /**
+   * Get a RWCursor for the last n bytes of this IOBufQueue.
+   *
+   * @methodset Accessors
+   */
   template <CursorAccess access>
   RWCursor<access> tail(size_t n) {
     RWCursor<access> result(
@@ -1285,6 +1718,11 @@ class QueueAppender : public detail::Writable<QueueAppender> {
     return result;
   }
 
+  /**
+   * Remove n bytes from the end of this IOBufQueue.
+   *
+   * @methodset Modifiers
+   */
   void trimEnd(size_t n) { queueCache_.queue()->trimEnd(n); }
 
  private:
