@@ -195,13 +195,14 @@ class AsyncSSLSocketConnector : public AsyncSocket::ConnectCallback,
 };
 
 AsyncSSLSocket::AsyncSSLSocket(
-    shared_ptr<SSLContext> ctx, EventBase* evb, Options&& options)
+    shared_ptr<const SSLContext> ctx, EventBase* evb, Options&& options)
     : AsyncSocket(evb),
       server_{options.isServer},
       ctx_{std::move(ctx)},
       certificateIdentityVerifier_{std::move(options.verifier)},
       handshakeTimeout_{this, evb},
-      connectionTimeout_{this, evb} {
+      connectionTimeout_{this, evb},
+      tlsextHostname_{std::move(options.serverName)} {
   init();
   if (options.isServer) {
     SSL_CTX_set_info_callback(
@@ -213,7 +214,7 @@ AsyncSSLSocket::AsyncSSLSocket(
 }
 
 AsyncSSLSocket::AsyncSSLSocket(
-    std::shared_ptr<folly::SSLContext> ctx,
+    std::shared_ptr<const folly::SSLContext> ctx,
     AsyncSocket::UniquePtr oldAsyncSocket,
     Options&& options)
     : AsyncSocket(std::move(oldAsyncSocket)),
@@ -221,7 +222,8 @@ AsyncSSLSocket::AsyncSSLSocket(
       ctx_{std::move(ctx)},
       certificateIdentityVerifier_{std::move(options.verifier)},
       handshakeTimeout_{this, AsyncSocket::getEventBase()},
-      connectionTimeout_{this, AsyncSocket::getEventBase()} {
+      connectionTimeout_{this, AsyncSocket::getEventBase()},
+      tlsextHostname_{std::move(options.serverName)} {
   noTransparentTls_ = true;
   init();
   if (options.isServer) {
@@ -237,7 +239,9 @@ AsyncSSLSocket::AsyncSSLSocket(
  * Create a client AsyncSSLSocket
  */
 AsyncSSLSocket::AsyncSSLSocket(
-    shared_ptr<SSLContext> ctx, EventBase* evb, bool deferSecurityNegotiation)
+    shared_ptr<const SSLContext> ctx,
+    EventBase* evb,
+    bool deferSecurityNegotiation)
     : AsyncSocket(evb),
       ctx_(std::move(ctx)),
       handshakeTimeout_(this, evb),
@@ -252,7 +256,7 @@ AsyncSSLSocket::AsyncSSLSocket(
  * Create a server/client AsyncSSLSocket
  */
 AsyncSSLSocket::AsyncSSLSocket(
-    shared_ptr<SSLContext> ctx,
+    shared_ptr<const SSLContext> ctx,
     EventBase* evb,
     NetworkSocket fd,
     bool server,
@@ -275,7 +279,7 @@ AsyncSSLSocket::AsyncSSLSocket(
 }
 
 AsyncSSLSocket::AsyncSSLSocket(
-    shared_ptr<SSLContext> ctx,
+    shared_ptr<const SSLContext> ctx,
     AsyncSocket* oldAsyncSocket,
     bool server,
     bool deferSecurityNegotiation)
@@ -296,7 +300,7 @@ AsyncSSLSocket::AsyncSSLSocket(
 }
 
 AsyncSSLSocket::AsyncSSLSocket(
-    shared_ptr<SSLContext> ctx,
+    shared_ptr<const SSLContext> ctx,
     AsyncSocket::UniquePtr oldAsyncSocket,
     bool server,
     bool deferSecurityNegotiation)
@@ -309,7 +313,7 @@ AsyncSSLSocket::AsyncSSLSocket(
  * to be sent in Client Hello.
  */
 AsyncSSLSocket::AsyncSSLSocket(
-    const shared_ptr<SSLContext>& ctx,
+    const shared_ptr<const SSLContext>& ctx,
     EventBase* evb,
     const std::string& serverName,
     bool deferSecurityNegotiation)
@@ -322,7 +326,7 @@ AsyncSSLSocket::AsyncSSLSocket(
  * and allow tlsext_hostname to be sent in Client Hello.
  */
 AsyncSSLSocket::AsyncSSLSocket(
-    const shared_ptr<SSLContext>& ctx,
+    const shared_ptr<const SSLContext>& ctx,
     EventBase* evb,
     NetworkSocket fd,
     const std::string& serverName,
@@ -582,7 +586,8 @@ void AsyncSSLSocket::sslAccept(
   checkForImmediateRead();
 }
 
-void AsyncSSLSocket::attachSSLContext(const std::shared_ptr<SSLContext>& ctx) {
+void AsyncSSLSocket::attachSSLContext(
+    const std::shared_ptr<const SSLContext>& ctx) {
   // Check to ensure we are in client mode. Changing a server's ssl
   // context doesn't make sense since clients of that server would likely
   // become confused when the server's context changes.
@@ -653,7 +658,7 @@ void AsyncSSLSocket::detachSSLContext() {
 
 #if FOLLY_OPENSSL_HAS_SNI
 void AsyncSSLSocket::switchServerSSLContext(
-    const std::shared_ptr<SSLContext>& handshakeCtx) {
+    const std::shared_ptr<const SSLContext>& handshakeCtx) {
   CHECK(server_);
   if (sslState_ != STATE_ACCEPTING) {
     // We log it here and allow the switch.
@@ -1181,10 +1186,13 @@ bool AsyncSSLSocket::willBlock(
     }
 #endif
 
-    // The timeout (if set) keeps running here
+    // the timeout (if set) keeps running here
     return true;
   } else {
+    // The error queue might contain multiple errors. We only consider the head.
+    // Clear the rest.
     unsigned long lastError = *errErrorOut = ERR_get_error();
+    ERR_clear_error();
     VLOG(6) << "AsyncSSLSocket(fd=" << fd_ << ", "
             << "state=" << state_ << ", "
             << "sslState=" << sslState_ << ", "
@@ -1580,10 +1588,16 @@ AsyncSocket::ReadResult AsyncSSLSocket::performReadSingle(
       // SSL_R_UNEXPECTED_EOF_WHILE_READING. We should then explicitly check for
       // that. See https://www.openssl.org/docs/man1.1.1/man3/SSL_get_error.html
       if (error == SSL_ERROR_SYSCALL && local_errno == 0) {
+        // ignore anything else in the error queue
+        ERR_clear_error();
         // intentionally returning EOF
         return ReadResult(0);
       }
+
+      // The error queue might contain multiple errors. We only consider and
+      // return the head. Clear the rest.
       auto errError = ERR_get_error();
+      ERR_clear_error();
       VLOG(6) << "AsyncSSLSocket(fd=" << fd_ << ", "
               << "state=" << state_ << ", "
               << "sslState=" << sslState_ << ", "
@@ -1701,7 +1715,10 @@ AsyncSocket::WriteResult AsyncSSLSocket::interpretSSLError(int rc, int error) {
         WRITE_ERROR,
         std::make_unique<SSLException>(SSLError::INVALID_RENEGOTIATION));
   } else {
+    // The error queue might contain multiple errors. We only consider and
+    // return the head. Clear the rest.
     auto errError = ERR_get_error();
+    ERR_clear_error();
     VLOG(3) << "ERROR: AsyncSSLSocket(fd=" << fd_ << ", state=" << int(state_)
             << ", sslState=" << sslState_ << ", events=" << eventFlags_ << "): "
             << "SSL error: " << error << ", errno: " << errno
