@@ -39,9 +39,11 @@
 #include <folly/Portability.h>
 #include <folly/ScopeGuard.h>
 #include <folly/Synchronized.h>
+#include <folly/container/F14Map.h>
 #include <folly/container/F14Set.h>
 #include <folly/executors/DrivableExecutor.h>
 #include <folly/executors/IOExecutor.h>
+#include <folly/executors/QueueObserver.h>
 #include <folly/executors/ScheduledExecutor.h>
 #include <folly/executors/SequencedExecutor.h>
 #include <folly/experimental/ExecutionObserver.h>
@@ -133,7 +135,8 @@ class EventBase : public TimeoutManager,
                   public DrivableExecutor,
                   public IOExecutor,
                   public SequencedExecutor,
-                  public ScheduledExecutor {
+                  public ScheduledExecutor,
+                  public GetThreadIdCollector {
  public:
   friend class ScopedEventBaseThread;
 
@@ -307,6 +310,17 @@ class EventBase : public TimeoutManager,
       timerTickInterval = interval;
       return *this;
     }
+
+    /**
+     * Controls the behavior of isInEventBaseThread(). See the method
+     * documentation for the semantics of this option.
+     */
+    bool strictLoopThread{false};
+
+    Options& setStrictLoopThread(bool b) {
+      strictLoopThread = b;
+      return *this;
+    }
   };
 
   /**
@@ -654,13 +668,13 @@ class EventBase : public TimeoutManager,
   template <typename T>
   void runInEventBaseThreadAndWait(void (*fn)(T*), T* arg) noexcept;
 
-  /*
+  /**
    * Like runInEventBaseThread, but the caller waits for the callback to be
    * executed.
    */
   void runInEventBaseThreadAndWait(Func fn) noexcept;
 
-  /*
+  /**
    * Like runInEventBaseThreadAndWait, except if the caller is already in the
    * event base thread, the functor is simply run inline.
    */
@@ -668,20 +682,20 @@ class EventBase : public TimeoutManager,
   void runImmediatelyOrRunInEventBaseThreadAndWait(
       void (*fn)(T*), T* arg) noexcept;
 
-  /*
+  /**
    * Like runInEventBaseThreadAndWait, except if the caller is already in the
    * event base thread, the functor is simply run inline.
    */
   void runImmediatelyOrRunInEventBaseThreadAndWait(Func fn) noexcept;
 
-  /*
+  /**
    * Like runInEventBaseThread, but runs function immediately instead of at the
    * end of the loop when called from the eventbase thread.
    */
   template <typename T>
   void runImmediatelyOrRunInEventBaseThread(void (*fn)(T*), T* arg) noexcept;
 
-  /*
+  /**
    * Like runInEventBaseThread, but runs function immediately instead of at the
    * end of the loop when called from the eventbase thread.
    */
@@ -732,21 +746,48 @@ class EventBase : public TimeoutManager,
   }
 
   /**
-   * wait until the event loop starts (after starting the event loop thread).
+   * Wait until the event loop starts (after starting the event loop thread).
    */
   void waitUntilRunning();
 
   size_t getNotificationQueueSize() const;
 
+  /**
+   * Returns the number of loop callbacks pending execution. If this is
+   * non-zero, loopOnce() is guaranteed to run the callbacks without blocking.
+   */
+  size_t getNumLoopCallbacks() const;
+
   void setMaxReadAtOnce(uint32_t maxAtOnce);
 
   /**
-   * Verify that current thread is the EventBase thread, if the EventBase is
-   * running.
+   * Verify that current thread is the EventBase thread.
+   *
+   * The definition of the EventBase thread depends on the strictLoopThread
+   * option.
+   *
+   * When the loop is running, isInEventBaseThread() returns true if and only if
+   * the current thread is the thread that is running the loop.
+   * Otherwise,
+   *
+   * - In default mode (strictLoopThread = false), isInEventBaseThread() always
+   *   returns true. This is to support use cases in which the loop is run
+   *   manually, and other EventBase methods can be interleaved with loop
+   *   runs. In this mode, if the loop is not running continuously it is
+   *   responsibility of the caller to ensure that all methods that may run
+   *   non-thread-safe logic (including, for example,
+   *   runImmediatelyOrRunInEventBaseThread*()) are serialized with loop runs.
+   *
+   * - In strict mode (strictLoopThread = true), isInEventBaseThread() always
+   *   returns false. This is to support use cases in which the loop is run by a
+   *   dedicated executor, possibly not continuously, so it is safe to rely on
+   *   isInEventBaseThread() from any thread with with no risk of races. In this
+   *   mode, the behavior is equivalent to inRunningEventBaseThread().
    */
   bool isInEventBaseThread() const {
     auto tid = loopThread_.load(std::memory_order_relaxed);
-    return tid == std::thread::id() || tid == std::this_thread::get_id();
+    return tid == std::this_thread::get_id() ||
+        (!strictLoopThread_ && tid == std::thread::id());
   }
 
   bool inRunningEventBaseThread() const {
@@ -869,6 +910,13 @@ class EventBase : public TimeoutManager,
    */
   std::thread::id getLoopThreadId();
 
+  /**
+   * Returns the timepoint at the start of the loop callbacks.
+   */
+  std::chrono::steady_clock::time_point getLoopCallbacksStartTime() {
+    return startWork_;
+  }
+
   /// Implements the Executor interface
   void add(Cob fn) override { runInEventBaseThread(std::move(fn)); }
 
@@ -907,6 +955,9 @@ class EventBase : public TimeoutManager,
   /// Implements the IOExecutor interface
   EventBase* getEventBase() override;
 
+  /// Implements the GetThreadIdCollector interface
+  WorkerProvider* getThreadIdCollector() override;
+
   static std::unique_ptr<EventBaseBackendBase> getDefaultBackend();
 
  protected:
@@ -928,6 +979,7 @@ class EventBase : public TimeoutManager,
 
  private:
   class FuncRunner;
+  class ThreadIdCollector;
 
   folly::VirtualEventBase* tryGetVirtualEventBase();
 
@@ -957,8 +1009,15 @@ class EventBase : public TimeoutManager,
   void initNotificationQueue();
 
   // Tick granularity to wheelTimer_
-  std::chrono::milliseconds intervalDuration_{
+  const std::chrono::milliseconds intervalDuration_{
       HHWheelTimer::DEFAULT_TICK_INTERVAL};
+  const bool enableTimeMeasurement_;
+  bool strictLoopThread_;
+
+  // The ID of the thread running the main loop. std::thread::id{} if loop is
+  // not running, otherwise acts as lock to enforce loop mutual exclusion.
+  std::atomic<std::thread::id> loopThread_;
+
   // should only be accessed through public getter
   HHWheelTimer::UniquePtr wheelTimer_;
 
@@ -977,10 +1036,6 @@ class EventBase : public TimeoutManager,
   // to determine if it should exit
   std::atomic<bool> stop_;
 
-  // The ID of the thread running the main loop.
-  // std::thread::id{} if loop is not running.
-  std::atomic<std::thread::id> loopThread_;
-
   // A notification queue for runInEventBaseThread() to use
   // to send function requests to the EventBase thread.
   std::unique_ptr<EventBaseAtomicNotificationQueue<Func, FuncRunner>> queue_;
@@ -1006,20 +1061,10 @@ class EventBase : public TimeoutManager,
   // callback called when latency limit is exceeded
   Func maxLatencyCob_;
 
-  // Enables/disables time measurements in loopBody(). if disabled, the
-  // following functionality that relies on time-measurement, will not
-  // be supported: avg loop time, observer and max latency.
-  const bool enableTimeMeasurement_;
-
   // Wrap-around loop counter to detect beginning of each loop
   std::size_t nextLoopCnt_;
   std::size_t latestLoopCnt_;
   std::chrono::steady_clock::time_point startWork_;
-  // Prevent undefined behavior from invoking event_base_loop() reentrantly.
-  // This is needed since many projects use libevent-1.4, which lacks commit
-  // b557b175c00dc462c1fce25f6e7dd67121d2c001 from
-  // https://github.com/libevent/libevent/.
-  bool invokingLoop_{false};
 
   // Observer to export counters
   std::shared_ptr<EventBaseObserver> observer_;
@@ -1035,7 +1080,8 @@ class EventBase : public TimeoutManager,
   friend class detail::EventBaseLocalBase;
   template <typename T>
   friend class EventBaseLocal;
-  folly::F14FastMap<std::size_t, erased_unique_ptr> localStorage_;
+  using LocalStorageMap = folly::F14FastMap<std::size_t, erased_unique_ptr>;
+  LocalStorageMap localStorage_;
   folly::Synchronized<folly::F14FastSet<detail::EventBaseLocalBase*>>
       localStorageToDtor_;
   bool tryDeregister(detail::EventBaseLocalBase&);
@@ -1045,6 +1091,8 @@ class EventBase : public TimeoutManager,
 
   // pointer to underlying backend class doing the heavy lifting
   std::unique_ptr<EventBaseBackendBase> evb_;
+
+  std::unique_ptr<ThreadIdCollector> threadIdCollector_;
 };
 
 template <typename T>
