@@ -28,7 +28,6 @@
 #include <folly/Conv.h>
 #include <folly/Portability.h>
 #include <folly/Range.h>
-#include <folly/String.h>
 #include <folly/Unicode.h>
 #include <folly/Utility.h>
 #include <folly/lang/Bits.h>
@@ -408,9 +407,13 @@ struct Input {
     return range_.subpiece(0, 16 /* arbitrary */).toString();
   }
 
-  [[noreturn]] dynamic error(char const* what) const {
+  [[noreturn]] void error(char const* what) const {
     throw json::make_parse_error(lineNum_, context(), what);
   }
+  template <typename R>
+  R error(char const* what) const {
+    error(what);
+  }
 
   json::serialization_opts const& getOpts() { return opts_; }
 
@@ -582,7 +585,8 @@ dynamic parseNumber(Input& in) {
   auto extremaLen = negative ? minIntLen : maxIntLen;
   auto extremaStr = negative ? minIntStr : maxIntStr;
   if (*in != '.' && !wasE) {
-    if (LIKELY(!in.getOpts().double_fallback || integral.size() < extremaLen) ||
+    if (FOLLY_LIKELY(
+            !in.getOpts().double_fallback || integral.size() < extremaLen) ||
         (integral.size() == extremaLen && integral <= extremaStr)) {
       auto val = to<int64_t>(integral);
       in.skipWhitespace();
@@ -618,7 +622,7 @@ void decodeUnicodeEscape(Input& in, std::string& out) {
         c >= '0' && c <= '9' ? c - '0' :
         c >= 'a' && c <= 'f' ? c - 'a' + 10 :
         c >= 'A' && c <= 'F' ? c - 'A' + 10 :
-        (in.error("invalid hex digit"), 0));
+        in.error<uint16_t>("invalid hex digit"));
     // clang-format on
   };
 
@@ -735,7 +739,7 @@ dynamic parseValue(Input& in, json::metadata_map* map) {
       in.consume("NaN") ?
         (in.getOpts().parse_numbers_as_strings ? (dynamic)"NaN" :
           (dynamic)std::numeric_limits<double>::quiet_NaN()) :
-      in.error("expected json value");
+      in.error<dynamic>("expected json value");
   // clang-format on
 }
 
@@ -994,7 +998,7 @@ std::string stripComments(StringPiece jsonC) {
         break;
       case State::InString:
         if (s[0] == '\\') {
-          if (UNLIKELY(s.size() == 1)) {
+          if (FOLLY_UNLIKELY(s.size() == 1)) {
             throw std::logic_error("Invalid JSONC: string is not terminated");
           }
           result.push_back(s[0]);
