@@ -164,7 +164,7 @@ bool operator==(const MoveTester& o1, const MoveTester& o2) {
   return o1.s_ == o2.s_;
 }
 
-TEST(Expected, value_or_rvalue_arg) {
+TEST(Expected, valueOrRvalueArg) {
   Expected<MoveTester, int> ex = makeUnexpected(-1);
   MoveTester dflt = "hello";
   EXPECT_EQ("hello", ex.value_or(dflt));
@@ -190,7 +190,7 @@ TEST(Expected, value_or_rvalue_arg) {
   EXPECT_EQ("hello", dflt); // only moved if used
 }
 
-TEST(Expected, value_or_noncopyable) {
+TEST(Expected, valueOrNoncopyable) {
   Expected<std::unique_ptr<int>, int> ex{unexpected, 42};
   std::unique_ptr<int> dflt(new int(42));
   EXPECT_EQ(42, *std::move(ex).value_or(std::move(dflt)));
@@ -205,14 +205,14 @@ struct ExpectingDeleter {
   }
 };
 
-TEST(Expected, value_move) {
+TEST(Expected, valueMove) {
   auto ptr = Expected<std::unique_ptr<int, ExpectingDeleter>, int>(
                  in_place, new int(42), ExpectingDeleter{1337})
                  .value();
   *ptr = 1337;
 }
 
-TEST(Expected, dereference_move) {
+TEST(Expected, dereferenceMove) {
   auto ptr = *Expected<std::unique_ptr<int, ExpectingDeleter>, int>(
       in_place, new int(42), ExpectingDeleter{1337});
   *ptr = 1337;
@@ -974,4 +974,62 @@ TEST(Expected, TestUnique) {
       }));
 }
 
+struct ConvertibleNum {
+  /*implicit*/ operator Expected<int, E>() const { return num_; }
+  int num_;
+};
+
+TEST(Expected, TestChainedConversion) {
+  auto vs =
+      std::vector<Expected<ConvertibleNum, E>>{ConvertibleNum{.num_ = 137}};
+  for (Expected<int, E> v : vs) {
+    ASSERT_EQ(137, *v);
+  }
+}
+
+struct ConvertibleError {
+  struct E1 {};
+  struct E2 {};
+
+  /*implicit*/ operator E() const {
+    if (std::holds_alternative<E1>(err_)) {
+      return E::E1;
+    }
+    return E::E2;
+  }
+
+  std::variant<E1, E2> err_;
+};
+
+struct ConvertibleFromE {
+  /*implicit*/ ConvertibleFromE(E e) : is_e1(e == E::E1) {}
+  bool is_e1;
+};
+
+Expected<int, E> implicitConvertFromConvertibleError() {
+  return makeUnexpected(ConvertibleError{ConvertibleError::E1{}});
+}
+
+Expected<int, ConvertibleFromE> implicitConvertFromE() {
+  const auto& unexpected = makeUnexpected(E::E1);
+  // Convert by ref.
+  return unexpected;
+}
+
+TEST(Expected, ImplicitErrorConversion) {
+  static_assert(
+      std::is_convertible_v<Unexpected<ConvertibleError>&&, Expected<int, E>>);
+  static_assert(
+      std::is_convertible_v<Unexpected<E>&&, Expected<int, ConvertibleFromE>>);
+
+  {
+    auto e = implicitConvertFromConvertibleError();
+    EXPECT_EQ(e.error(), E::E1);
+  }
+  {
+    auto e = implicitConvertFromE();
+    EXPECT_EQ(e.error().is_e1, true);
+  }
+}
+
 } // namespace folly
