@@ -304,6 +304,63 @@ static_assert( //
 
 static_assert(std::is_nothrow_destructible<Function<int(int)>>::value, "");
 
+#if FOLLY_HAS_DEDUCTION_GUIDES
+
+struct ctor_guide {
+  static void fn();
+  static void fn_nx() noexcept;
+
+  struct call {
+    void operator()();
+  };
+  struct call_c {
+    void operator()() const;
+  };
+  struct call_nx {
+    void operator()() noexcept;
+  };
+  struct call_c_nx {
+    void operator()() const noexcept;
+  };
+};
+
+static_assert( //
+    std::is_same_v< //
+        Function<void()>,
+        decltype(Function{ctor_guide::fn})>);
+static_assert( //
+    std::is_same_v< //
+        Function<void()>,
+        decltype(Function{&ctor_guide::fn})>);
+static_assert( //
+    std::is_same_v< //
+        Function<void()>,
+        decltype(Function{ctor_guide::call{}})>);
+static_assert( //
+    std::is_same_v< //
+        Function<void() const>,
+        decltype(Function{ctor_guide::call_c{}})>);
+#if FOLLY_HAVE_NOEXCEPT_FUNCTION_TYPE
+static_assert( //
+    std::is_same_v< //
+        Function<void() noexcept>,
+        decltype(Function{ctor_guide::fn_nx})>);
+static_assert( //
+    std::is_same_v< //
+        Function<void() noexcept>,
+        decltype(Function{&ctor_guide::fn_nx})>);
+static_assert( //
+    std::is_same_v< //
+        Function<void() noexcept>,
+        decltype(Function{ctor_guide::call_nx{}})>);
+static_assert( //
+    std::is_same_v< //
+        Function<void() const noexcept>,
+        decltype(Function{ctor_guide::call_c_nx{}})>);
+#endif
+
+#endif
+
 struct RecStd {
   using type = std::function<RecStd()>;
   /* implicit */ RecStd(type f) : func(f) {}
@@ -354,7 +411,7 @@ TEST(Function, InvokeReference) {
 // TEST =====================================================================
 // Emptiness
 
-TEST(Function, Emptiness_T) {
+TEST(Function, EmptinessT) {
   Function<int(int)> f;
   EXPECT_EQ(f, nullptr);
   EXPECT_EQ(nullptr, f);
@@ -1037,7 +1094,7 @@ TEST(Function, ConvertReturnType) {
 // TEST =====================================================================
 // asStdFunction_*
 
-TEST(Function, asStdFunction_void) {
+TEST(Function, asStdFunctionVoid) {
   int i = 0;
   folly::Function<void()> f = [&] { ++i; };
   auto sf = std::move(f).asStdFunction();
@@ -1048,7 +1105,7 @@ TEST(Function, asStdFunction_void) {
   EXPECT_EQ(1, i);
 }
 
-TEST(Function, asStdFunction_void_const) {
+TEST(Function, asStdFunctionVoidConst) {
   int i = 0;
   folly::Function<void() const> f = [&] { ++i; };
   auto sf = std::move(f).asStdFunction();
@@ -1059,7 +1116,7 @@ TEST(Function, asStdFunction_void_const) {
   EXPECT_EQ(1, i);
 }
 
-TEST(Function, asStdFunction_return) {
+TEST(Function, asStdFunctionReturn) {
   int i = 0;
   folly::Function<int()> f = [&] {
     ++i;
@@ -1073,7 +1130,7 @@ TEST(Function, asStdFunction_return) {
   EXPECT_EQ(1, i);
 }
 
-TEST(Function, asStdFunction_return_const) {
+TEST(Function, asStdFunctionReturnConst) {
   int i = 0;
   folly::Function<int() const> f = [&] {
     ++i;
@@ -1087,7 +1144,7 @@ TEST(Function, asStdFunction_return_const) {
   EXPECT_EQ(1, i);
 }
 
-TEST(Function, asStdFunction_args) {
+TEST(Function, asStdFunctionArgs) {
   int i = 0;
   folly::Function<void(int, int)> f = [&](int x, int y) {
     ++i;
@@ -1101,7 +1158,7 @@ TEST(Function, asStdFunction_args) {
   EXPECT_EQ(1, i);
 }
 
-TEST(Function, asStdFunction_args_const) {
+TEST(Function, asStdFunctionArgsConst) {
   int i = 0;
   folly::Function<void(int, int) const> f = [&](int x, int y) {
     ++i;
@@ -1118,7 +1175,7 @@ TEST(Function, asStdFunction_args_const) {
 // TEST =====================================================================
 // asSharedProxy_*
 
-TEST(Function, asSharedProxy_void) {
+TEST(Function, asSharedProxyVoid) {
   int i = 0;
   folly::Function<void()> f = [&i] { ++i; };
   auto sp = std::move(f).asSharedProxy();
@@ -1129,7 +1186,7 @@ TEST(Function, asSharedProxy_void) {
   EXPECT_EQ(2, i);
 }
 
-TEST(Function, asSharedProxy_void_const) {
+TEST(Function, asSharedProxyVoidConst) {
   int i = 0;
   folly::Function<void() const> f = [&i] { ++i; };
   auto sp = std::move(f).asSharedProxy();
@@ -1140,7 +1197,7 @@ TEST(Function, asSharedProxy_void_const) {
   EXPECT_EQ(2, i);
 }
 
-TEST(Function, asSharedProxy_return) {
+TEST(Function, asSharedProxyReturn) {
   folly::Function<int()> f = [i = 0]() mutable {
     ++i;
     return i;
@@ -1151,7 +1208,7 @@ TEST(Function, asSharedProxy_return) {
   EXPECT_EQ(2, spcopy());
 }
 
-TEST(Function, asSharedProxy_return_const) {
+TEST(Function, asSharedProxyReturnConst) {
   int i = 0;
   folly::Function<int() const> f = [&i] {
     ++i;
@@ -1163,7 +1220,7 @@ TEST(Function, asSharedProxy_return_const) {
   EXPECT_EQ(2, spcopy());
 }
 
-TEST(Function, asSharedProxy_args) {
+TEST(Function, asSharedProxyArgs) {
   int i = 0;
   folly::Function<int(int, int)> f = [&](int x, int y) mutable {
     ++i;
@@ -1177,7 +1234,7 @@ TEST(Function, asSharedProxy_args) {
   EXPECT_EQ(2, i);
 }
 
-TEST(Function, asSharedProxy_args_const) {
+TEST(Function, asSharedProxyArgsConst) {
   int i = 0;
   folly::Function<int(int, int) const> f = [&i](int x, int y) {
     ++i;
@@ -1189,18 +1246,18 @@ TEST(Function, asSharedProxy_args_const) {
   EXPECT_EQ(562, spcopy(5, 6));
 }
 
-TEST(Function, asSharedProxy_nullptr) {
+TEST(Function, asSharedProxyNullptr) {
   auto sp = folly::Function<int(int, int) const>::SharedProxy(nullptr);
   EXPECT_THROW(sp(3, 4), std::bad_function_call);
 }
 
-TEST(Function, asSharedProxy_empty) {
+TEST(Function, asSharedProxyEmpty) {
   auto func = folly::Function<int(int, int) const>();
   auto sp = std::move(func).asSharedProxy();
   EXPECT_THROW(sp(3, 4), std::bad_function_call);
 }
 
-TEST(Function, asSharedProxy_explicit_bool_conversion) {
+TEST(Function, asSharedProxyExplicitBoolConversion) {
   folly::Function<void(void)> f = []() {};
   auto sp = std::move(f).asSharedProxy();
   auto spcopy = sp;
@@ -1229,7 +1286,7 @@ std::array<int, 3> badCopierF(BadCopier a, const BadCopier& b, BadCopier&& c) {
   c.v_ *= -1;
   return ret;
 }
-TEST(Function, asSharedProxy_forwarding) {
+TEST(Function, asSharedProxyForwarding) {
   folly::Function<decltype(badCopierF)> ff(badCopierF);
   EXPECT_TRUE((std::is_same_v<
                decltype(ff),
@@ -1355,7 +1412,7 @@ TEST(Function, CtorWithCopy) {
   EXPECT_FALSE(noexcept(Function<void()>(ly)));
 }
 
-TEST(Function, Bug_T23346238) {
+TEST(Function, BugT23346238) {
   const Function<void()> nullfun;
 }
 
@@ -1425,3 +1482,47 @@ TEST(Function, TrivialSmallBig) {
   EXPECT_EQ(7, s2());
   EXPECT_EQ(7, h2());
 }
+
+TEST(Function, ConstInitEmpty) {
+  static FOLLY_CONSTINIT Function<int()> func;
+  EXPECT_THROW(func(), std::bad_function_call);
+}
+
+TEST(Function, ConstInitNullptr) {
+  static FOLLY_CONSTINIT Function<int()> func{nullptr};
+  EXPECT_THROW(func(), std::bad_function_call);
+}
+
+TEST(Function, ConstInitStaticLambda) {
+  static FOLLY_CONSTINIT Function<int()> func{[] { return 3; }};
+  EXPECT_EQ(3, func());
+}
+
+namespace {
+template <typename T>
+union consteval_immortal {
+  T value;
+  template <typename... A>
+  explicit FOLLY_CONSTEVAL consteval_immortal(folly::in_place_t, A&&... a)
+      : value{static_cast<A&&>(a)...} {}
+  ~consteval_immortal() {}
+};
+} // namespace
+
+TEST(Function, ConstEvalEmpty) {
+  static FOLLY_CONSTINIT consteval_immortal<Function<int()>> func{
+      folly::in_place};
+  EXPECT_THROW(func.value(), std::bad_function_call);
+}
+
+TEST(Function, ConstEvalNullptr) {
+  static FOLLY_CONSTINIT consteval_immortal<Function<int()>> func{
+      folly::in_place, nullptr};
+  EXPECT_THROW(func.value(), std::bad_function_call);
+}
+
+TEST(Function, ConstEvalStaticLambda) {
+  static FOLLY_CONSTINIT consteval_immortal<Function<int()>> func{
+      folly::in_place, [] { return 3; }};
+  EXPECT_EQ(3, func.value());
+}
