@@ -201,14 +201,14 @@ TEST(SimpleSubprocessTest, ChangeChildDirectoryWithError) {
   }
 }
 
-TEST(SimpleSubprocessTest, waitOrTerminateOrKill_waits_if_process_exits) {
+TEST(SimpleSubprocessTest, waitOrTerminateOrKillWaitsIfProcessExits) {
   Subprocess proc(std::vector<std::string>{"/bin/sleep", "0.1"});
   auto retCode = proc.waitOrTerminateOrKill(1s, 1s);
   EXPECT_TRUE(retCode.exited());
   EXPECT_EQ(0, retCode.exitStatus());
 }
 
-TEST(SimpleSubprocessTest, waitOrTerminateOrKill_terminates_if_timeout) {
+TEST(SimpleSubprocessTest, waitOrTerminateOrKillTerminatesIfTimeout) {
   Subprocess proc(std::vector<std::string>{"/bin/sleep", "60"});
   auto retCode = proc.waitOrTerminateOrKill(1s, 1s);
   EXPECT_TRUE(retCode.killed());
